var Backup = {errors:{}};   // Holder for the backup methods.

/***********
Backup.copyContact
Make a copy of the contact record in a catalogue item. There are some restrictions at the 
moment! Primary among theres is that the meta-data *must* fit in 7000 octets! The function
returns 'true' if the copy is successfully made. The 'catParent' is the integer catalogue 
nodeID of the parent under which to make the copy.
***********/
Backup.copyContact = function(contactRec, catParent) {

   // Remove the meta-data so the record fits in the itemData field.
   var metaData = contactRec.customData;
   delete contactRec.customData;

   var metaDataStr = uneval(contactRec);

   if (metaDataStr.length > 6500) {
      // Too big - can't update this one!!!!!
      return false;
   }

   // Create the catalogue item.
   var catObj = {
     code:        contactRec.uName,
     description: contactRec.name,
     itemData:    contactRec,
     stockData:   metaData
   }
   // Create the catalogue item...
   var catId = rb.catalogue.item.create(catObj);
   if (catId<0)
      // couldn't!
      return false;

   // ...  and link under the parent provided.
   return rb.catalogue.category.linkItem(catId, catParent)>=0;
}
/***********
Backup.restoreContact
Take a contact backup from the catalogue. The key is the 'code' value which
maps to the uName in contact. If the uName exists then restore the information.
If it is not then re-create the nName.

If the record already exists AND the update flag is true then replace the 
existing record.
***********/
Backup.restoreContact = function(catRec, update) {
   // build the contact record.
   var cont = eval(catRec.itemData);
   if (cont==null)
      return false;

   // Add in the metaData.
   cont.customData = eval(catRec.stockData);

   // Now - try and create the contact record. Assume it doesn't exist!
   var res = rb.contact.createIndividual(cont);

   if (res<0) {
      // This item already exists. 
      if (update) {
         // Do a modify instead of create. To do this we need the ID of the existing contact.
         var contIt = rb.contact.get([], {uName:cont.uName});
         if (contIt.getNextRow()) {
            // Got the existing record.
            cont.ID = contIt.ID;
            return rb.contact.modifyIndividual(cont);
         }
      }
      // already exists so don't restore.
      return false;
   }
   return true;
}

/***********
Backup.copyCat
Make a copy of the catalogue record under a catalogue node. The copy will also 
include copies of any files that exist under the old item.
***********/
Backup.copyCat = function(catRec, catParent) {
   dumpObj(catRec, "catRec");

   WRITE_DEB('</h1>Backup.copyCat - START</h1>\n');

   // First - create a copy of any files under this item.
   var files    = catRec.fileRefs;
   var newFiles = [];
   var idx;
   if (files!=null) {
      for (idx=0;idx<files.length;idx++) {
         var aFile = files[idx];

         // Read the existing file.
         var fileIt = rb.file.get(aFile);
         if (fileIt.getNextRow()) {
            // Got the file. Make a copy.
            var newFileId = rb.file.create({name:fileIt.name, owner:fileIt.owner, description:fileIt.description, fileDef:fileIt.fileDef});

            if (newFileId<0) {
               Backup.errors.push("couldn't create copy of file.");
               return false;
            }
            // Made the copy - add to the newFiles array.
            newFiles.push({fileOrder:aFile.fileOrder, fileID:newFileId});
         }
      }
   }
   catRec.fileRefs = newFiles;

   WRITE_DEB("<h4>Writing catalogueItem</h4>n");
   dumpObj(catRec, 'catRec');
   catRec.itemID = null;
   delete catRec.itemID;
   var catId = rb.catalogue.item.create(catRec);

   if (catId<0) {
      Backup.errors.push('Failed to create catalogue copy.');
      return false;
   }
   WRITE_DEB('<h4>New catId is : '+catId+'</h4>\n');

   // Made the copy - have to locate it under the parent.
   try {
      return rb.catalogue.category.linkItem(catId, catParent)>=0;
   }
   catch (ev) {
      return false;
   }
}
/********
Backup.deleteAll
Delete all catalogue items (and files) under the specified ID.
********/
Backup.deleteAll = function(parentId) {
var search  = {idMkr:"N", ID:parentId, allLevels:1};
var srcCat  = rb.catalogue.item.get(search);
var idArray = [];

   while (srcCat.getNextRow()) {
      // Delete the files.
      WRITE_DEB('Found itemID='+srcCat.itemID+'<br/>\n');
      idArray.push(srcCat.itemID);

      var files = srcCat.fileRefs;
      if (files!=null) {
         for (var i=0;i<files.length;i++) {
            dumpObj(files[i], "files["+i+"]");
            // Delete this file.
            try {
               rb.file.remove(files[i].fileID);
            }
            catch (ev) {
                  ;
            }
         }
      }
   }
   dumpObj(idArray, 'idArray');
   for (var i=0;i<idArray.length;i++) {
      WRITE_DEB('Deleting cat item: itemID='+idArray[i]+'<br/>');
      rb.catalogue.item.remove(idArray[i]);
   }
}

/***********
Backup.restoreCat
Take a catalogue item and copy it under the specified parent. Also restores all files
under the parent.
***********/
Backup.restoreCat = function(catRec, catParent) {
   var catId = rb.catalogue.item.create(catRec);
   return catId>=0;
}
