/**************************************************************************************
Utility function that takes a pathname and splits
it into an array or tokens. Returns a JavaScript
array of tokens - which may be empty!
**************************************************************************************/
function SplitPath(path,delim) {
  if (delim==null)
    delim='/';
	if( path == null || typeof path != "string" ) {
		rb.debug.write("SplitPath = "+path.toSource()+"<br>\n");
		return [];
	}
	var arr = path.split(delim);
	if( arr[0] == "" )
		arr.shift();
	return arr;
}

/**************************************************************************************
Function to take a 'path' and return the item referenced. If the
path does NOT resolve to something that exists then the function 
returns 'null'. If the item is located - then it is returned 
to the client.

leafType is one of:
    'N' - catalogue node.
    'I' - catalogue item.
    'C' - contact (not supported yet)

NOTE - this function assumes that each element in the path is
unique. It does not attempt to deal with the case where there 
are multiple catalogue entries with the same name with the
same parent.
**************************************************************************************/
function ResolvePath(path, leafType, delim) {
var token = SplitPath(path, delim);

   if (token.length <=0 )
      // Path is empty or invalid.
      return null;

   // Now - search for each token in the path.
   var parentId = 0; // Start from the root of the catalogue.
   var it;
   var search;

   // Look around.
   var i, getType = 'N';

   for (i=0;i<token.length;i++) {
      if ((i+1)==token.length) {
         // Accessing the leaf. What kind of request should we make?
         // From the current parent - search for child items that match the token.
         // rb.debug.write("<h1>At leaf node - setting type to "+leafType);
         getType = leafType;
      }
      switch (getType) {
      case 'N':
         // Category.
         search = {idMkr:'N', ID:parentId, keyword:[token[i]], nextLevel:true, searchInName:true};
         it = rb.catalogue.category.get(search);
         if (!it.getNextRow()) {
            // Not found. Return null.
            // rb.debug.write("<p>Not Found</p>");
            return null;
         }

         parentId = it.nodeID;
         break;
      case 'I':
         // Item.
         search = {idMkr:'N', ID:parentId, keyword:[token[i]], allLevels:0, searchInCode:true};
         var it = rb.catalogue.item.get(search);

         if (!it.getNextRow())
            // Not found. Return null.
            return null;

         parentId = it.itemID;
         break;
      default:
         // Contact.
         rb.debug.error("Unsupported leaf-type. in ResolvePath.");
         break;                                                   
      }
   }
   return it;
}


/**************************************************************************************
Define a function to take a name, parent name and description. The function
creates a new catalogue 'category'. If the 'path' parameter is null then
the new category is a child of the virtual root. If the path is specified
then there are TWO choices. If the parameter is a number then make the new category 
a child of the specified ID - if it exists. If it is a string then resolve
the path to find the parent - and attach it accordingly.
***************************************************************************************/
function MakeCategory(categoryData, path) {
   // First thing to do is find the nodeID of the parent. By 
   // default this is zero (0) - the root of the tree.
   var nodeId   = -1;
   var it;

   categoryData.parentID = 0;

   if (path!=null) {
      rb.debug.write("<p>Path specified : "+path+"</p>\n");
      if (typeof path == "number")
         categoryData.parentID = path;
      else {
         // Treat the path as a string - in which case it must be the name 
         // of a category that is a child of the root.
         it = ResolvePath(path, 'N');
         if (it != null)
            categoryData.parentID = it.nodeID;
         else {
            // Path NOT resolved. Error and return.
            rb.debug.error("Can't resolve path for new category. Path is ["+path+"]");

            return -1;
         }
      }
   }
   // Does a child of the parent exist with the name specified? If so then DO NOT create!
   rb.debug.write("<h1>parentId = "+categoryData.parentID+"</h1>");

   var searchData = {idMkr:'N', ID:categoryData.parentID, keyword:[categoryData.name], nextLevel:true, searchInName:true};
   it = rb.catalogue.category.get(searchData);
   if (!it.getNextRow()) {
      // Doesn't exist - so create it.
      rb.debug.write("<h2>MakeCategory("+uneval(categoryData)+', '+path+');');
   	nodeId = rb.catalogue.category.create(categoryData);
   }
   else {
      // Category already exists - just return the node ID.
      rb.debug.write("<p>Category already exists : "+categoryData.toSource()+"</p>");
      nodeId = it.nodeID;
   }

   return nodeId;
}

/**************************************************************************************
MakeItem
Create a catalogue 'item'. It should be appended under the named parent element. The 
parent is either named by it's numeric nodeID - by by specifying a path.
***************************************************************************************/
function MakeItem(itemData, path, update) {
var nodeId   = -1;
var it;
var parentId;

   // First thing to do is find the nodeID of the parent. By 
   // default this is zero (0) - the root of the tree.

   parentId = 0;

   if (path!=null) {
      rb.debug.write("<p>Path specified : "+path+"</p>\n");
      if (typeof path == "number")
         parentId = path;
      else {
         // Treat the path as a string - in which case it must be the name 
         // of a category that is a child of the root.
         it = ResolvePath(path, 'N');
         if (it != null)
            parentId = it.nodeID;
         else {
            // Path NOT resolved. Error and return.
            rb.debug.error("Can't resolve path for new category. Path is ["+path+"]");

            return -1;
         }
      }
   }
   // Does a child of the parent exist with the name specified? If so then DO NOT create!
   rb.debug.write("<h1>parentId = "+parentId+"</h1>");

   var searchData = {idMkr:'N', ID:parentId, keyword:[itemData.code], searchInCode:true, nextLevel:true, searchInName:true};
   it = rb.catalogue.item.get(searchData);
   if (!it.getNextRow()) {
      // Doesn't exist - so create it.
      rb.debug.write("<p>Item does NOT exist. Creating catalogue item.</p>");
      rb.debug.write("<h2>MakeCategory("+uneval(itemData)+', '+path+'); - creating NEW item');

	   if (itemData.description==null)
		   itemData.description = "";
	   
	   if (itemData.itemData==null)
		   itemData.itemData = new Object;

   	nodeId          = rb.catalogue.item.create(itemData);
      itemData.itemID = nodeId;

      // If there is a parent specified - link this item under that parent.
      if (parentId>0)
         rb.catalogue.category.linkItem(nodeId, parentId);
   }
   else {
      // Category already exists - if 'update is defined then copy over  parameters and write back.
      if (update==true) {
         rb.debug.write("<h1>Item already exists : "+itemData.toSource()+" - UPDATING</h1>");
         for (var data in itemData) {
            it[data] = itemData[data];
         }
         rb.catalogue.item.modify(it);
      }
      else
         rb.debug.write("<p>Item already exists : "+itemData.toSource()+" - NO CHANGES</p>");
      itemData = it;
   }

   return itemData;
}
