/*******************
Doc
********************
Wrapper class for generic documents. Each news article is a catalogue item. If the
length of the article body is <Doc.maxItemLength characters it is stored in the stockData field - 
otherwise it is in a separate file.
********************/
var Config;
if (Config==null)
   Config = {};

var SysData;
if (SysData==null)
   SysData={};

function Doc(input_data, andContent) {
var restored = false;
var it = null;

   this.customData = {};
   this.privData   = {};
   this.mime       = 'text/html';
   this.state      = 'Draft';    // Other state values are defined by the document type (subclass)
   this.ID         = -1;
   this.category   = -1;         // where it should eventually reside (as well as root).
   this.hidden     = false;      // 'true' if this document should *not* be in the doc archive when released.
   
   // Identify which file to use. 'fileIndex' is used as an index into the fileDefs array within a catalogue item.
   // Most doc-types leave this as zero .Things like webpages (for example) use index=1 for the draft version.
   this.fileID     = -1;         // Where to find the content for this document.
   this.fileIndex  = 0;          
   this.fileRefs   = [];
   
   this.created    = new Date();
   this.content    = {};

   this.writeAccess  = '';        // Will default to the author before saving.
   this.readAccess   = '';        // this is the default - and means everyone.

   if (Config.defaultPermissions!=null) {
      this.writeAccess  = Config.defaultPermissions.writeAccess;
      this.readAccess   = Config.defaultPermissions.readAccess;
   }

   // Set to the access permissions of the current user.
   this.assocParents = [];
   this.assocCommunities = [];    // Contact communities we want this stored under.
   this.rb$type    = this.docType;

   if (input_data == null)
      input_data = {};
   
   var id       = (input_data.ID == null)?-1:input_data.ID;
   var authorID = (input_data.authorID==null)?-1:input_data.authorID;

   // If there is an ID then try to read this catalogue item.
   if (id>0)
      var it = rb.catalogue.item.get({ID:id, idMkr:'I', allLevels:false});
   else if (authorID>0)
      // Might be under the author - a contact template ID.
      var it = rb.catalogue.item.get({ID:authorID, idMkr:"C", allLevels:false});

   if (it!=null) {
      if (it.getNextRow() && Doc.isInstance(it)) {
         restored = true;

         this.makeDoc(it);
         // If the content is required then read it now.
         if (andContent==null || andContent==true)
           this.getContent();

         for (var p=0;p<Doc.props2.length;p++) {
           if (it[Doc.props2[p].item]!=null)
              this[Doc.props2[p].news] = it[Doc.props2[p].item];
        }
      }
   }
   // Overlay the input properties.
   for (var p in Doc.props3) {
     if (input_data[p]!=null)
       this[p] = input_data[p];
   }
   // Deal with assocParent array.
   if (input_data.assocParents!=null) {
     if (typeof input_data.assocParents == 'object')
       this.assocParents = input_data.assocParents;
     else
       this.assocParents = [input_data.assocParents];
   }
   // And with the communties.
   if (input_data.assocCommunities!=null) {
     if (typeof input_data.assocCommunities == 'object')
       this.assocCommunities = input_data.assocCommunities;
     else
       this.assocCommunities = [input_data.assocCommunities];
   }
}

Doc.maxFileLength = 1024*100;


// Doc.createUrl
// This method takes a URL pattern and subsitutes document ID and other values into the string, returning the result.
// Substitution paramters are 
//   '$id'    - the document ID
//   '$title' - title field

Doc.createUrl = function(baseStr, doc) {
  var workUrl = new String(baseStr);

  workUrl = workUrl.replace(/\$id/,doc.ID);
  workUrl = workUrl.replace(/\$title/,doc.title);
  return workUrl;
}

Doc.prototype.validateImages = function() {
var res;

  // Check for web-images.
  // All the envelope information is OK. Validate the HTML content itself.
  rb.debug.write("<h1>IN validateImages</h1>");
  var aParser = new XmlParser;
  aParser.mode(XmlModeHTML);
  this.getContent();
  var contentStr = (this.content.data).toString();
  if (aParser.build(contentStr)==null && aParser.errors.length>0) {
    res = "The HTML content for the document is invalid. Correct HTML errors and try again ("+aParser.errors.length+")<br><p align=left>"+Util.doXmp(contentStr)+'<br>\n';
    for (var i=0;i<aParser.errors.length;i++) {
        rb.page.write(Util.doXmp(aParser.errors[i])+'<br>\n');
    }
  }
  else if (ImgProc!=null) {
    // OK - the HTML is valid. Resolve images within the document.
    this.unres = ImgProc.parse(aParser, aParser.root, this.ID);
    if (this.unres!=null) {
      res = "Document contains unresolved images.";
    }
    else {
      // *No* unresolved images - but the HTML may have been modified.
      this.content.data = new Binary;
      this.content.data.set(aParser.root.execute().text());
    }
  }
  return res;
}
// Extract the content parameters from the parameter set provided. The object contains
// on of 3 things (if more than one then the first detected is used!):
//    dataset.content        - string containing the raw input for the document.
//           .content_server - string - reference to a file on this server.
//            content_upload - a binary object representing this document.
// the result is 'this.content' represents the content of the document. The data
// presented is validated as well as maybe - if it cannot be used then the method
// returns an error string. If the data is OK then the method returns 'null'.
Doc.prototype.validateContent = function(dataset, errObj) {   // Overlay the generic rb$params parts.
  var content = {type:'none', loc:'', mime:''};
  var mime    = 'text/html';
  var res;

  rb.debug.write("<h4>dataset==null = "+(dataset==null)+"</h4>\n");

  if (dataset==null) {
     rb.page.write("<h1>AM HERE - -XYZ<br>");
     return (this.mime=='text/html')?this.validateImages() : true;
  }

  if (errObj==null)
    errObj = {};
 
  dumpObj(this, "DOCUMENT");
  dumpObj(dataset, "DATASET");

  var valImages = (this.mime=='text/html');

  if (dataset.content_server && dataset.content_server.length>0) {
    // Referencing a file on the server - this argument is the file name.
    valImages = false;
    content.type = 'server';
    content.loc  = dataset.content_server;
    mime = dataset.content_server_mime;
  }
  else if (dataset.content_upload!=null && dataset.content_upload.length>0) {
    // Work out the last element in the path.
    rb.debug.write('<h4>Have uploaded content...</h4>\n');
    var tmpPathArr = (dataset.content_upload.fname()).split(/[\/\\]/);
    var tmpPath = tmpPathArr.pop()
      
    content.type = 'stored';
    content.data = dataset.content_upload; // a binary object
    content.loc  = tmpPath;
    // rb.page.write("<h2>Content fname is ["+dataset.content_upload.fname()+"], path="+tmpPath+", length is "+dataset.content_upload.length+"</h2>\n");

    mime = dataset.content_upload.mime();
    var valImages = (mime=='text/html');
  }
  else if (dataset.content!=null && dataset.content!='') {
    // Is it a simple string?
    if (dataset.content.length>Doc.maxFileLength)
      return 'Uploaded file is too big. Maxium size is approx '+(Doc.maxFileLength/1024)+'Kbytes';

    content.type = 'stored';
    content.data = new Binary;
    content.data.set(dataset.content);
  }
  else {
    // No content uploaded - leave same as previous version.
    rb.debug.write("<h1>CONTENT UNCHANGED</h1>\n");
    content = this.content;
    mime    = this.mime;
  }
  if (dataset.pubDate) {
    // Specifying a publication date - validate and convert to a date object.
    this.pubDate = Util.parseDate(dataset.pubDate);
    if (this.pubDate==null) {
      errObj.pubDate = true;
      return 'Invalid publication date';
    }
    rb.debug.write("<h1>pubDate is "+this.pubDate.toString()+"</h1>\n");
  }
  // rb.page.write("Max file length = "+Doc.maxFileLength+"<br>");
  if (content.data!=null && content.data.length>Doc.maxFileLength) {
    errObj.content = true;
    return 'Uploaded file is too big. Maxium size is approx '+(Doc.maxFileLength/1024)+'Kbytes';
  }
  
  // Got this far - it's OK!
  this.content = content;
  this.mime    = mime;

  if (valImages)
    res = this.validateImages();

  return res;
}

Doc.flags = {};

// getContent
// This method returns a binary object representing the content of the file. If the file is stored
// locally then the binary object includes the data. If the file is stored remotely then the binary 
// object includes the url/fname of the target.
Doc.prototype.getContent = function() {
var res;
  if (this.ID>0) {
    if (Util.isBinary(this.content.data)) {
      // Been called before - just return the data I already have.
      rb.debug.write("<h1>data already loaded - returning</h1>\n");
      return this.content.data;
    }
    if (this.content.type=="stored") {
      // We have the content in a file object - load the object and use it.
      if (this.fileID>=0) {
        var fileIt = rb.file.get({fileID:this.fileID});
        if (fileIt.getNextRow()) {
          if (Util.isBinary(fileIt.fileDef))
            this.content.data = fileIt.fileDef;
          else {
            this.content.data = new Binary;
            this.content.data.set(fileIt.fileDef);
          }
        }
      }
    }
    else if (this.content.type=='server') {
      // File on the local server. Create a shell binary object that will be loaded on demand (if required by the application).
      this.content.data = new Binary;
      this.content.data.fname(this.content.loc);
      this.content.data.load();
    }
  }
}

Doc.prototype.makeDoc = function (item) {

   // Got the existing article. Load the contents.
   var dataSource;

   if (item.itemData!=null) {
     if (item.itemData.rb$type)
       this.rb$type = item.itemData.rb$type;
     else
       this.rb$type = 'unknown'

     dataSource = this.privData = item.itemData.privData;
     this.customData = item.itemData.customData;
   }

   for (var p=0;p<Doc.props2.length;p++) {
      // rb.debug.write("<h4>1 - looking for '"+Doc.props2[p].item+"'...");
     if (item[Doc.props2[p].item]!=null)
         this[Doc.props2[p].news] = item[Doc.props2[p].item];
      else
         delete this[Doc.props2[p].news];
   }
   if (item.fileRefs && item.fileRefs.length>this.fileIndex)
      this.fileID = item.fileRefs[this.fileIndex].fileID;
   else
      this.fileID = -1;

   if (dataSource!=null) {
      for (var p in Doc.props) {
         if (dataSource[p]!=null)
            this[p] = dataSource[p];
         else
            delete this[p];
      }
      // Check for custom properties from a subclass.
      if (this.customprops!=null) {
        for (var p in this.customprops) {
           if (dataSource[p]!=null)
              this[p] = dataSource[p];
           else
              delete this[p]
        }
      }
   }

   // Finally - if there is a state then save it so we can detect state changes on save.
   this.oldState = this.state;
} 

// Used between catalogue and Doc
// a) Things from the itemData record.
Doc.props   = { "title":1, 
                "authorID":1,   // catalogue/member ID of author.
                "author":1, 
                "created":1,    // Date item was created.
                "lastEdit":1,   // Date of last '.save()' operation
                "pubDate":1,    // Official publication date. Determines order in release lists etc. If not specified - defaults to 'created'.
                "visibility":1, // string: one of 'public', 'restricted'
                "mime":1, 
                "state":1, 
                "parent":1,     // catalogue ID of the catagory under which this item should be placed when published.
                "assocParents":1,// An array of additional parents to which this document should be attached.
                "assocCommunities":1,// Array of contact communities we want to reside under.
                "status":1,
                "readAccess":1,
                "writeAccess":1,
                "summary":1,
                "content":1
              };

// b) Things from the top level of the catalogue item.
Doc.props2  = [   {news:'title',     item:'description'},
                  {news:'keywords',  item:'keywords'},
                  {news:'clientData',item:'stockData'},
                  {news:'ID',        item:'itemID'},
                  {news:'fileRefs',  item:'fileRefs'}];

// Used between rb$params and Doc
Doc.props3  = {title:1, authorID:1, author:1, created:1, pubDate:1, lastEdit:1, visibility:1, mime:1, summary:1, parent:1, state:1, readAccess:1, writeAccess:1, keywords:1};
Doc.prototype.docType = '/doc'	
Doc.prototype.getDocType = function() {return this.rb$type;}

/*****************************************************************
Doc.isInstance
*****************************************************************/
Doc.isInstance = function (catItem) {
    return (catItem.itemData != null && catItem.itemData.rb$type!= null && (catItem.itemData.rb$type.indexOf(Doc.prototype.docType)==0));
}

// calcKey()
// generate a 'name' key for this event such that when sorted in name order - new items appear first.
// this is done by making the key some date in the future minus the event date. The key is based on the
// 'pubDate' - if specified, which allows it to be modified manually - for example if publication date 
// changes.
Doc.prototype.calcKey = function() {
  var now = (this.pubDate!=null ? this.pubDate.valueOf() : this.created.valueOf());
  var end = new Date(2030, 12, 31);

  if (isNaN(now)) {
    now = new Date;
    now = now.valueOf();
  }

  var key = (end.valueOf()-now);
  return rb.page.format("%014.0f",key);
}
/********************************************************************************
Doc.prototype.makeItem()
Create data needed to be saved for this article. Returns an object with TWO
things in it.
   item:{}    - Catalogue item. Including ID if available.
   file:{}    - File object *if* article is too big for the catalogue.
********************************************************************************/
Doc.prototype.makeItem = function () {
  var item = {itemData:{rb$type:this.getDocType()}, stockData:{}, fileRefs:[]};
  
  if (this.rb$type!=null)
    item.rb$type = this.rb$type;
  
  if (this.customData)
    item.itemData.customData = this.customData;
  
  if (this.privData==null)
    this.privData = {};
  
  // Copy top level properties.
  for (var p in Doc.props) {
    // rb.debug.write('Testing property "'+Doc.props[p]+'...');
    if (this[p] != null) {
      // rb.debug.write('Copying "'+this[Doc.props[p]]+'"<br>');
      this.privData[p] = this[p];
    }
  }
  // Check for custom properties from a subclass.
  if (this.customprops!=null) {
    for (var p in this.customprops) {
      if (this[p]!=null)
        this.privData[p] = this[p];
    }
  }
  var limit = ((this.fileRefs==null || this.fileIndex>this.fileRefs.length) ? this.fileIndex : this.fileRefs.length);

  for (var i=0;i<limit;i++) {
    if (this.fileRefs[i]==null) {
      rb.debug.write("<h1>this.fileRefs["+i+"] = "+uneval(this.fileRefs[i])+"</h1>\n");
  
      // Have a null reference. Need to create an empty file to store here.
      var emptyFile = new Binary;
      emptyFile.set(' ');
      var fileID = rb.file.create({fileDef:emptyFile, name:'empty', owner:'empty', description:''});
      rb.debug.write("<h1>Created empty file - ID is "+fileID+"</h1>\n");

      this.fileRefs[i] = {fileOrder:i, fileID:fileID};
    }
  }

  // Copy top level properties.
  for (var p=0;p<Doc.props2.length;p++) {
    if (this[Doc.props2[p].news]!=null)
      item[Doc.props2[p].item] = this[Doc.props2[p].news];
  }
  item.itemData.privData   = this.privData;
  item.itemData.customData = this.customData;
  item.code = this.calcKey();

  return item;
}


/********************************************************************************
Doc.prototype.createContent
Set up to 'fileRefs' for a document before it is created.
********************************************************************************/
Doc.prototype.createContent = function(item) {

  if (this.content.type=='stored' && this.content.data) {
    var fileData = {fileDef:this.content.data, name:'document', owner:this.author};
    var fileID = rb.file.create({fileDef:this.content.data, name:'document', owner:this.author});
    if (fileID>=0)
       item.fileRefs[this.fileIndex]={fileOrder:this.fileIndex, fileID:fileID};
  }
}

/********************************************************************************
Doc.prototype.create
Use the data in the Doc object to create a new category item to represent
the new article. The function returns -1 if it fails. The method checks the size 
of the content and write the content to *either* a file or the stock data area.
********************************************************************************/
Doc.prototype.create = function() {
var item;
var draft = (this.state=='Draft')

   if (this.authorID==null) {
      // MUST have a valid author ID!
      rb.page.write("<h2>NO USER ID - trying to save document</h1>");
      return -1;
   }

   if (this.writeAccess==null || this.writeAccess=='') {
     this.writeAccess = extauth.sessionUser;
   }

   // Set the modify time to 'now'.
   this.lastEdit =
   this.created  = new Date();

   if (this.pubDate==null)
     this.pubDate = this.created;

   if (this.content==null)
     this.content=''

   item = this.makeItem();

   this.createContent(item);
   
   // Now save the item, reference it under the 'News' root and return the ID.
   delete item.itemID;

   this.ID = rb.catalogue.item.create(item);

   if (this.ID>=0) {
      // Link it to the author.
      rb.contact.linkCatItem(this.authorID, this.ID);

      // Link it to the Docs items if it is to be published.
      if (!draft)
        this.publish();
   }
   return this.ID;
}

Doc.prototype.publish = function() {
   if (this.parent!=null) {
     try {
       rb.catalogue.category.linkItem(this.ID, this.parent);
     }
     catch (ev) {
       ;
     }
   }
   // *all* document classes are stored under the documents node of the tree as 
   // well as any other home they may have, uless they have the hidden attribute set. 
   // The document node has  two further children - one for 'Released' (formal 
   // documents) and one for 'obsolete' decide where we should put this one - and 
   // unlink from the others.
   if (!this.hidden) {
     var docRoot = SysData.RootCats.Documents;
 
     if (this.state == 'Released')
       docRoot = SysData.RootCats.Released;
     else if (this.state == 'Obsolete')
       docRoot = SysData.RootCats.ArchDocs;

     rb.debug.write("<h1>Linking to document home at node="+docRoot+"</h1>");
    
     // Link to the new home.
     try {
       rb.catalogue.category.linkItem(this.ID, docRoot);
     }
     catch (ev) {;}

     // And unlink from any others.
     if (docRoot!= SysData.RootCats.Documents)
       rb.catalogue.category.unlinkItem(this.ID, SysData.RootCats.Documents);

     if (docRoot!= SysData.RootCats.Released)
       rb.catalogue.category.unlinkItem(this.ID, SysData.RootCats.Released);

     if (docRoot!= SysData.RootCats.ArchDocs)
       rb.catalogue.category.unlinkItem(this.ID, SysData.RootCats.ArchDocs);
   }
   if (this.newAssocParents!=null) {
      // Been provided with a new set of parents. Make necessary changes.
      rb.debug.write("<h1>HERE - 1</h1>\n");
      var assocParents = this.assocParents;
      var newAssocParents = this.newAssocParents;

      var testOld = {};
      var testNew = {};
      var addTo   = []; 
      var remFrom = [];

      for (var a=0;a<assocParents.length;a++)
         testOld[assocParents[a]] = true;

      for (var a=0;a<newAssocParents.length;a++)
         testNew[newAssocParents[a]] = true;

      for (var a=0;a<assocParents.length;a++) {
         if (testNew[assocParents[a]] == null)
            remFrom.push(assocParents[a]);
      }
      for (var a=0;a<newAssocParents.length;a++) {
         if (testOld[newAssocParents[a]] == null)
            addTo.push(newAssocParents[a]);
      }

      // Make the changes.
      for (var a=0;a<remFrom.length;a++)
         try {rb.catalogue.category.unlinkItem(this.ID, remFrom[a]);} catch (ev) {;}

      for (var a=0;a<addTo.length;a++)
         try {rb.catalogue.category.linkItem(this.ID, addTo[a]);} catch (ev) {;}

      this.assocParents = newAssocParents;
   }
   else if (this.assocParents!=null) {
     for (var i=0;i<this.assocParents.length;i++) {
       // Link to this catagory.
       try {rb.catalogue.category.linkItem(this.ID, this.assocParents[i]);} catch (ev) {;}
     }
   }
   // Finally there may be an array of other place where we want to link this element.
   if (this.assocCommunities!=null) {
     for (var i=0;i<this.assocCommunities.length;i++) {
       // Link to this catagory.
       try {rb.contact.linkCatItem(this.assocCommunities, this.ID);} catch (ev) {;}
     }
   }

   return true;
}

/********************************************************************************
Doc.prototype.save
Save the contents of the article after an edit. Reuse a file if one has already
been used - otherwise only float to a file if the content is now too big to
fit in the stockdata field of the catalogue item structure.
********************************************************************************/
Doc.prototype.save = function () {
  var draft = (this.state=='Draft')
    var item;

  if (this.ID==null || this.ID<0) {
    rb.debug.error("Attempt to save a Doc with no ID attribute - probably not created!");
    return false;
  }
  
  this.lastEdit = new Date();
  item = this.makeItem();
  
  // Is there already a file ID?
  var contentSaved = false;
  
  if (this.content.type=='stored') {
    if (item.fileRefs.length>this.fileIndex && item.fileRefs[this.fileIndex].fileID!=null && item.fileRefs[this.fileIndex].fileID>0) {
      rb.debug.write("<h4>Reusing existing file container.</h4>\n");

      // Have a file already - reuse it.
      var fileIt = rb.file.get({fileID:item.fileRefs[this.fileIndex].fileID, allData:1});
      if (fileIt.getNextRow()) {
        // OK - file already exists. Change content and save.
        fileIt.fileDef = this.content.data
        if (this.mime.length>20)
          fileIt.fileDef.mime('application/unknown');

        if (rb.file.modify(fileIt)<0)
          return false;
        
        contentSaved = true;
      }
    }
    if (!contentSaved) {
      // Need a file. Create file.
      var fileID = rb.file.create({fileDef:this.content.data, name:'document', owner:this.author});
      if (fileID>=0)
        item.fileRefs[this.fileIndex]={fileOrder:this.fileIndex, fileID:fileID};
    }
  }
  // Now save the Doc itself.
  if (this.newAssocParents) {
     item.itemData.privData.assocParents = this.newAssocParents;
  }

  var res = (rb.catalogue.item.modify(item)>0);
  if (res && !draft)
    this.publish();
  
  return res;
}

/********************************************************************************
Doc.prototype.remove
Remove news item. Delete file if there is one.
********************************************************************************/
Doc.prototype.remove = function () {
var fileID;

   if (this.ID==null || this.ID<0) {
      rb.debug.error("Attempt to save a Doc with no ID attribute - probably not created!");
      return false;
   }

   if (this.fileRef.length>0 && this.fileRef[0].fileID!=null && this.fileRef[0].fileID>0)
      // File exists. Remember the ID and delete when we've successfully deleted the item.
      fileID = this.fileRef[0];

   if (rb.catalogue.item.remove(this.ID)>0) {
      if (fileID)
         rb.file.remove(fileID);
   }

   return true;
}

/********************************************************************************
DocIt - Doc Iterator.
This is a new JavaScript class - it isn't directly instantiated by the application
instead being returned from the Event.get class method. The class 'looks' exactly
like an iterator - but behind the scenes does the manipulation to extract contact
info into the member structure.

Because an iterator is basically a subclass of the basic data structure - with the
addition of the iterator methods. In JavaScript we achieve this simply by setting
the prototype to be an instance of Event.
 ********************************************************************************/
/********************************************************************************
DocIt
Iterator for Docs information. This is the constructor. Doesn't do anything 
except at the moment.
********************************************************************************/
function DocIt(catIterator) {
    this.rb$Iterator = catIterator;
}

/********************************************************************************
* Force the iterator to be a subclass of the Event.
*********************************************************************************/
DocIt.prototype = new Doc;

/********************************************************************************
Doc.prototype.checkAccess
Given the currently loaded document - check whether it can be accessed by the
current user. Return an object containing the permissions:
  {write:bool, read:bool}
Note that write implies read -someone with write access can also read the 
document.

The 'alsoAllow' parameter allows the caller to specify a set of users or classes
of users that are allowed to access to this document regardless of there
presence in the access list.
********************************************************************************/
Doc.prototype.checkAccess = function (alsoAllow) {

  if (alsoAllow==null)
    alsoAllow = '';
  else
    alsoAllow += ',';

  if (extauth.sessionAuth) {
    // Test write access.
    if (extauth.isIn(alsoAllow+this.writeAccess))
       return {read:true, write:true};

    // No write access - check read access. If the read access string
    // is empty - this means it is a completely public document. Any
    // one can read it.
    if (this.readAccess.search(/[^\s]/)<0 || extauth.isIn(alsoAllow+this.readAccess))
       return {read:true, write:false};
  }
  else {
    // Allow public read-access if the readAccess list is empty.
    if (this.readAccess=='')
      return {read:true, write:false};
  }

  // No access
  return {read:false, write:false};
}
/********************************************************************************
Doc.checkAccessFromCatItem
Similar to the above routine - but can be used to check access against a catalogue
item before the document is created.
********************************************************************************/
Doc.checkAccessFromCatItem = function (item, alsoAllow) {

  if (alsoAllow==null)
    alsoAllow='';
  else
    alsoAllow+=',';

  // Get the access variables from the metadata.
  if (Doc.isInstance(item) && item.itemData!=null && item.itemData.privData!=null) {
    var data = item.itemData.privData;

    // Test write access.
    if (extauth.isIn(alsoAllow+data.writeAccess))
       return {read:true, write:true};

    // No write access - check read access. If the read access string
    // is empty - this means it is a completely public document. Any
    // one can read it.
    if (data.readAccess=='' || extauth.isIn(alsoAllow+data.readAccess))
       return {read:true, write:false};
  }
  // No access
  return {read:false, write:false};
}
/********************************************************************************
DocIt.prototype.getNextRow
Overload for the iterator getNextRow method.
********************************************************************************/
DocIt.prototype.getNextRow = function (wantWrite) {
var res = false;
var test = (wantWrite ? 'write' : 'read');

  do {
    do {
        res = this.rb$Iterator.getNextRow();
    } while (res && !Doc.isInstance(this.rb$Iterator));
    if (res) {
       this.makeDoc(this.rb$Iterator);
       var stattest = (this.rb$status==null || (this.rb$status).indexOf(this.state)>=0);
    }
  } while ((res==true && ((stattest==false) || (this.checkAccess('super')[test]==false))));
   
  return res;
}
/********************************************************************************
Doc.get
Create an iterator for Docs.
********************************************************************************/
Doc.get = function (data) {
var iterator;
var memIt;

   if (data==null)
      data = {ID:SysData.RootCats.Documents, idMkr:'N', allLevels:1};

   iterator = rb.catalogue.item.get(data);
   docIt = new DocIt(iterator);
   docIt.rb$status = data.status;
   return docIt;
}
