<!-- rbm:insertpage macro implementation. This tag takes
     a path attribute and uses these to identify 
     a catalogue stored we-page document. If the document 
     is not located in the database then the system looks 
     for a 'default' tag that identifies the default text 
     for this page.

     Default values are:
      name AND id missing - uses the name of the file being executed.
      default missing - empty page.

     If the 'draft' attribute has a value of 'yes' then the page inserts 
     the current working version of the file - should one exist.
-->
<rb:script src="/include/glossary.js"/>

<rb:macrotag name="rbm:insertpage">
  var content;
  var params         = rb.page.attributes();
  var pageTitle      = params.title;
  var Banner         = params.banner;
  var keywords       = params.keywords;
  var editAccessList = params.defaultWriters;  // Default access list.
  var thisPage;

  if (editAccessList==null)
    editAccessList = 'super,webadmin,web';

  // *all* insertpage content may be modified.
  var enablePageEdit = true;

  var wantDraft = (params.draft!=null && params.draft=='yes');

  if (content==null) {
    // Decide on the default access list for this page, if there is no database document.
    var readAccess = (params.access==null || params.access=='' || (extauth.sessionAuth && extauth.isIn(params.access)));

    // First time called - try to load the content.
    // a) Try to locate a database page with this name.
    var pageName = rb.page.env.Path();

    // b) Is there a 'name' attribute to override this?
    if (params.path!=null)
      pageName = params.path;

    // c) We think we know the page to load - see whether it exists by trying 
    //    to get it from the template.
    var thisPage = WebPage.getByPath(pageName, false, wantDraft, true, null, true);

    var content = '';

    if (thisPage != null) {
      // Exists *and* we have access to the page.
      readAccess = thisPage.canRead;

      editAccessList = thisPage.writeAccess;

      thisPage.getContent();
      if (thisPage)
        content = thisPage.content.data.toString();

      var tmpPageTitle = thisPage.getPageTitle(wantDraft);
      if (tmpPageTitle!=null && tmpPageTitle!='')
        pageTitle = tmpPageTitle;
      
      if (thisPage.keywords!=null && thisPage.keywords!='')
        keywords = thisPage.keywords;

      // Content came from database. Do the necessary transformations to make it look correct.
      var newParser = new XmlParser(rb.xml);

      newParser.build(content);
      content = newParser.root.execute().text();
    }
    if (!readAccess) {
      if (extauth.sessionAuth)
        content = "<h2>Access denied</h2>\n<p>You do not have sufficient access rights to read this page.</p>\n"
      else
        content = "<h2>Access denied</h2>\n<p>This page is protected. You must log into the system to see the contents.</p>\n"
    }
    if (content=='' || content==' ' || true) {
      // Page doesn't exist. Is there a 'default' attribute pointing us to a file?
      if (params['default']!=null) {
        var fileData = new Binary;
        if (fileData.load(params['default'])) {
          content = fileData.toString();
        }
      }

      if (content.length<=1) {
        if (params.defaultText!=null)
          content = params.defaultText;
        else
          // Last chance - output the body of this tag.
          content = rb.xml.thisTag().execute().bodytext();
      }
    }
  }
  else 
      editAccessList = thisPage ? thisPage.writeAccess : 'super,webadmin,web';
  
  if (thisPage && content.length>1) {
    var tmpBanner = thisPage.getPageBanner(wantDraft);
    if (tmpBanner!=null)
      Banner = tmpBanner;
  }

  // Do the standard glossary checks and output.
  // rb.page.write(txlateGlossary(content));
  if (params.show=='yes')
    rb.page.write(content);
  else {
    // We're being invoked in the header. Need to write the page title and meta-tags.
    rb.page.write('<title>'+pageTitle+'</title>');
    if (keywords)
      rb.page.write('<meta name="keywords" content="'+keywords+'">\n');
  }
</rb:macrotag>
