<!-- ************************************************************************

     This file contains a number of macros and functions for building a 
     presentation page with a top banner and a menu banner from a pattern page. 

     ************************************************************************ -->

<rb:script>
  var rb$menuBullets = new Array;
  var rb$menuItems = new Array;     // was MenuAnchors
  var rb$menuTree = new Array;      // was MenuLinks
  var rb$autoSelect = null;         // the auto-selected menu item.
  var rb$keyedURL = rb.page.env.Path();
  if (rb.page.env.QueryString())
    rb$keyedURL += '?'+rb.page.env.QueryString();
  var rb$pageAtts = new Object;

  function rbgettagatts(tag){
	if(typeof tag == "object" && tag.attributes!=null)
	   return tag.attributes;
	return tag;
  }
</rb:script>


<rb:macrotag name="rbm:selectmenu">
  // Overrides the auto-selected menu item.

  var key = rb.page.attribute("key");
  if (!key)
    rb.debug.write('&lt;rbm:selectmenu&gt;: Name/key not supplied in selectmenu tag.<br>\n');
  else
    rbselectmenuitem(key, true);
</rb:macrotag>

<rb:macrotag name="rbm:selectmenuitem">
  // Overrides the auto-selected menu item.

  var key = rb.page.attribute("key");
  if (!key)
    rb.debug.write('&lt;rbm:selectmenuitem&gt;: Name/key not supplied in selectmenu tag.<br>\n');
  else
    rbselectmenuitem(key, true);
</rb:macrotag>

<rb:script>
  function rbselectmenuitem(key, select) {
    var item = rb$menuItems[key];
    if (item==null) 
      rb.debug.write('rbselectmenuitem(): Menu item key "'+key+'" cannot be found.<br>\n');
    else
      if ((null == select) || (true == select))
      {
        item.lenurl = 999;  // to force selection!
        rb$autoSelect = item;
      }
      else
        rb$autoSelect = null;
  }
</rb:script>


<rb:macrotag name="rbm:item">
  // Attributes: 
  //   Only one of...
  //     parent     : parent menu item key/name
  //     name       : display name
  //     blockid    : id of rb:block
  //   +++ any of...
  //     key        : item key (defaults to name or blockid)
  //     href       : URI
  //     target     : 
  //     expand     : force menu tree expansion (if 'yes')

  // Adds menu tree to structure...
  rbaddmenutree(rb.xml.thisTag());
</rb:macrotag>

<rb:script>
  function rbaddmenulist(list) 
  {
    // Converts an array of menu definitions to a menu item tree.
    // Preferred to rbaddmenutree() for multi-level definitions.
    //
    // Example Usage: (rlevel is relative level within this menu (sub)tree)
    //    var list = new Array;
    //    list.push({parent:"Staff", rlevel:1};  // name/key of parent menu item
    //    list.push({name:"Aji Oyekanmi", key:"Aji", href:"aji.rhtm", rlevel:2};
    //    list.push({name:"Ashley Bill", key:"Ash", href:"ash.rhtm", rlevel:3};
    //    list.push({name:"Pete Wilson", key:"Pete", href:"pete.rhtm", rlevel:2};
    //    rbaddmenulist(list);  // attaches new menu items to the parent menu item

    var mtree = new Array;

    for (var i=0; i<list.length; i++)
    {
      if (list[i] && list[i].rlevel)
      {
        var mitem = new Array;
        for (var x in list[i])
          mitem[x] = list[i][x];
        if (0==mtree.length || mtree[mitem.rlevel-1])
        {
          mtree[mitem.rlevel] = mitem;
          if (mtree[mitem.rlevel-1])
            mtree[mitem.rlevel-1].push(mitem);
          mitem.rlevel = void 0;
        }
        else
          rb.debug.write('rbaddmenulist(): List item '+i+': '+list[i].toSource()+' has no parent.<br>\n');
      }
      else
        rb.debug.write('rbaddmenulist(): List item '+i+': '+list[i].toSource()+' has no rlevel.<br>\n');
    }
    return rbaddmenutree(mtree[list[0].rlevel]);
  }

  function rbaddmenutree(tag, level) {
    // Builds up a menu item tree from menu a tag tree.
    // Also creates a tree of top level items from which the menu is displayed.
    //
    // Example Usage:
    //    var tags = new Array;
    //    tags.parent = "Staff";  // name/key of parent menu item
    //    tags.push({name:"Aji Oyekanmi", key:"Aji", href:"aji.rhtm"};
    //    tags.push({name:"Pete Wilson", key:"Pete", href:"pete.rhtm"};
    //    rbaddmenutree( tags);  // attaches new menu items to the parent menu item

    var item;
    var att = rbgettagatts(tag);

    if (att) {
      // Parent locator or a new child?
      if (att.parent) {
        item = rb$menuItems[att.parent];
        if (!item) {
          // Parent node is missing!
          rb.debug.write('rbaddmenutree(): Parent key/name "'+att.parent+'" cannot be found.<br>\n');
          return;
        }
      }
      else
        if (att.name || att.blockid) 
        {
          item = new Array;    // Could have children!
          item.key = att.key ? att.key : (att.blockid ? att.blockid : att.name);
          item.doBullet = (att.bullet==null ||  att.bullet=='yes');
          item.level = level==null ? 1 : level;
          if (att.expand=='yes')
          {
            item.expand = true;
            if (!rb$menuTree.expandlev || (item.level > rb$menuTree.expandlev))
              rb$menuTree.expandlev = item.level;
          }

          if (!rb$menuTree.expandmax || (item.level > rb$menuTree.expandmax))
            rb$menuTree.expandmax = item.level;

          if (rb$menuItems[item.key]==null) // menu item key is unique
            rb$menuItems[item.key] = item;
//          else
//            rb.debug.error('rbaddmenutree(): Menu item name/key '+item.key+' redefinition.\n');

          if (1 == item.level)
            rb$menuTree.push(item);
        }
        else
        {
          rb.debug.error('ERROR - parent, name or blockid required in tag<br>\n');
          return;
        }

      if (att.blockid)
      {
        var menutags = rb.xml.root.findId(att.blockid)[0];
        item.id = att.blockid;
        item.name = menutags.execute().bodytext();
      }
      else
        if (att.name)
          item.name = att.name;

      if (att.target)
        item.target = att.target;

      if (att.href && att.href.length>0)
      { // find the menu item href that best matches the keyed url...
        item.href = att.href;
        var genhref;
        if ('?' == item.href[0])                   // only the query string is defined !!
          genhref = rb.page.env.Path()+item.href;  // prefix with parent's href ???
        else
          genhref = rb.page.abspath(item.href);

        if (rb$keyedURL.substr(0,genhref.length)==genhref && !item.expand)
          if ((null==rb$autoSelect) || (rb$autoSelect.lenurl <= genhref.length))
          {
            item.lenurl = genhref.length;
            rb$autoSelect = item;
          }
      }

      // build menu tree for any child tags...
      for (var child=0; child<tag.length; child++) {
        // Only interested in tags - not text.
		    if( tag[child].type=="tag" && tag[child].name=="rbm:item") {
          var childItem = rbaddmenutree(tag[child], item.level+1);
          childItem.parentkey = item.key;
          item.push( childItem );
          if (childItem.expand)
            item.expand = true;
        }
      }
    }
    return item;
  }
</rb:script>


<rb:macrotag name="rbm:menubullet">
  rbmenubullet(rb.xml.thisTag());
</rb:macrotag>

<rb:script>
  function rbaddmenubullet(tag)
  { return rbmenubullet(tag);}

  function rbmenubullet(tag)
  {
    var att = rbgettagatts(tag);
    if (att.level)
    {
      att.level = parseInt(att.level);
      var b = rb$menuBullets[att.level] ? rb$menuBullets[att.level] : new Object;
      for (var x in att)  // apply the new bullet attributes...
        b[x] = att[x];
      b.nimage = b.normal ? '<img src="'+b.normal+'" width="'+b.width+'" height="'+b.height+'" border="0"/>' : '';
      b.simage = b.select ? '<img src="'+b.select+'" width="'+b.width+'" height="'+b.height+'" border="0"/>': '';
      b.eimage = b.autoexp ? '<img src="'+b.autoexp+'" width="'+b.width+'" height="'+b.height+'" border="0"/>': b.nimage;
      b.aimage = b.across ? ((b.across.length>3) ? '<img src="'+b.select+'" width="'+b.width+'" height="'+b.height+'" border="0"/>' : b.across) : '';
      b.oimage = b.across1st ? ((b.across1st.length>3) ? '<img src="'+b.select+'" width="'+b.width+'" height="'+b.height+'" border="0"/>': b.across1st) : '';
      rb$menuBullets[b.level] = b;
    }
    else
      rb.debug.error('Warning: Menu bullet level missing<br>\n');
  }
</rb:script>

<rb:macrotag name="rbm:writemenu">
  var atts = rbgettagatts(rb.xml.thisTag());
  rbwritemenu(atts.level, atts.line);
</rb:macrotag>

<rb:script>
  function rbwritemenu(level, line) {
    // Identify the menu branch to be expanded for autoselection
    // and output the menu hierarchy...
    var mt = rb$menuTree;  // short hand!
    if (!mt || mt.length<=0)
      return;

    var mi = rb$autoSelect;
    // dumpObj(mi,"mi");
    while (mi) {  // identify the path to the auto selected item
      mi.autoexp = true;
      // dumpObj(mi,"mi");
      mi = rb$menuItems[mi.parentkey];
    }

    mt.show = {toplevel:1, endlevel:(mt.expandlev ? mt.expandlev+1 : 1), all:false, path:false, firstone:true, bwidth:[]};
    var ms = mt;  // assume we are going to show the menu tree with expanded nodes.

    if ("across"==line || "down"==line)
      mt.show[line] = true;
    else
      mt.show.tree = true;

    if (rb$autoSelect)
      switch(level)
      {
        case "*":  // path of selected item...
        case "all":
          mt.show.all = true;
          mt.show.endlevel = mt.expandmax;
          break;

        case "<":  // path of selected item...
        case "path":
          mt.show.path = true;
          mt.show.endlevel = rb$autoSelect.level;
          break;

        case ">":  // sub level of selected item...
        case "sub":
          mt.show.toplevel = rb$autoSelect.level + 1;
          mt.show.endlevel = mt.show.toplevel;
          ms = rb$autoSelect;
          break;

        case "=":  // level of selected item...
        case "sel":
          mt.show.toplevel = rb$autoSelect.level;
          mt.show.endlevel = mt.show.toplevel;
          if ((ms = rb$menuItems[rb$autoSelect.parentkey])==null)
           ms = mt;
          break;

        case "1":  // top level only...
        case "one":
          mt.show.endlevel = mt.show.toplevel;
          break;

        case "+":  // levels to selected item excluding subs...
        case "tosel":
          mt.show.endlevel = rb$autoSelect.level;
          break;

        case "-":  // levels to selected item including subs...
        case "tosub":
        default:
          if (mt.show.endlevel<=rb$autoSelect.level)
            mt.show.endlevel = rb$autoSelect.level + (rb$autoSelect.autoexp ? 1 : 0);
          break;
      }

    if (ms.length>0)        // Output appropriate menu (tree) into a table...
      rbwritemenulevel(ms, true, mt.show.across);
  }


  function rbwritemenulevel(item, tabp, rowp)
  {
    // Outputs menu items at one level...

    if (tabp)
      rb.page.write('<table border="0" cellpadding="0" cellspacing="0">\n'+(rowp ? '<tr valign="middle">\n' : ''));

    for (var child=0; child<item.length; child++)
      if (!rb$menuTree.show.path || item[child].autoexp)
        rbwritemenuitem(item[child]);

    if (tabp)
      rb.page.write((rowp ? '</tr>\n' : '')+'</table>\n');
  }


  function rbwritemenuitem(item)
  {
    // Outputs a menu item into a row in the table...

    var autosel = (item == rb$autoSelect);

    var bullet = rb$menuBullets[item.level];
    if (item.doBullet==false)
      bullet=null;
    else if (null==bullet)
      bullet = rb$menuBullets[0];
    var img = bullet 
      ? (rb$menuTree.show.across
        ? (rb$menuTree.show.firstone
          ? bullet.oimage
          : bullet.aimage
          ) 
        : (autosel 
          ? bullet.simage 
          : (item.autoexp 
              ? bullet.eimage 
              : bullet.nimage
            )
          )
        ) 
      : ''; 

    if (bullet && !rb$menuTree.show.bwidth[item.level])
      rb$menuTree.show.bwidth[item.level] = bullet.width;

    if (!rb$menuTree.show.across)
      rb.page.write('<tr valign="middle">\n');

    var textspan = 1;
    var spanWidth = 0;
    
    if (rb$menuTree.show.tree) { 
      textspan = rb$menuTree.show.endlevel + 1 - item.level;

      for (var i=rb$menuTree.show.toplevel; i<item.level; i++) {
        var wdth = parseInt(rb$menuTree.show.bwidth[i].valueOf());
        if (wdth==0)
          wdth=20;
        // dumpObj(wdth, "wdth");
        spanWidth += wdth;
      }
      rb.page.write('<td>');
      if (spanWidth>0)
        rb.page.write('<img src="/images/spacer.gif" width="'+(parseInt(spanWidth))+'" height="1"/>');
    }
    textspan += (null==item.id ? 0 : 1);

    if (null==item.id) {
//rb.page.write('<td colspan="'+(textspan)+'" height="'+bullet.vspace+'"');
//      rb.page.write('<td');
//      if (bullet)
//        rb.page.write(' width="'+bullet.width+'" height="'+bullet.vspace+'"');
//      rb.page.write('>');
      if (rb$menuBullets.style)
        rb.page.write('<span class="rbmenubullet'+bullet.level+(autosel?'sel':'')+'">&nbsp;</span>');
      else {
        if (img!='')
          rb.page.write(img);
        else
          rb.page.write('&nbsp;');
      }
//      rb.page.write('</td>\n');
    }
//    rb.page.write('<td align="left" colspan="'+(textspan)+'">');
    if (item.name)
    {
      if (rb$pageAtts.rbselmenulink!="yes" && item==rb$autoSelect)
        rb.page.write('<span class="menuselect">'+item.name+'</span>');
      else
        if (!item.href)
          rb.page.write('<span class="menunormal">'+item.name+'</span>');
        else
        {
          rb.page.write('<a class="'+(autosel ? "menuselect":"menu")+'" href="'+item.href+'"');
	  if (item.target)
            rb.page.write(' target="'+item.target+'"');
          rb.page.write('>'+item.name+'</a>');
        }
    }
    rb.page.write('</td>\n');
    if (!rb$menuTree.show.across)
      rb.page.write('</tr>\n');

    rb$menuTree.show.firstone = false;

    if ((item.length>0) && (item.level<rb$menuTree.show.endlevel) &&
        (item.expand || item.autoexp || rb$menuTree.show.all))
      rbwritemenulevel(item);  // display sub menu...
  }
</rb:script>


<rbm:menubullet level=0 normal="/images/bullet0.gif" select="/images/bullet0sel.gif" width="9" height="9"/>
<rbm:menubullet level=1 normal="/images/bullet0.gif" select="/images/bullet0sel.gif" width="9" height="9" vspace="25"/>
<rbm:menubullet level=2 normal="/images/bullet2.gif" select="/images/bullet2sel.gif" width="9" height="9" vspace="20"/>
<rbm:menubullet level=3 normal="/images/bullet3.gif" select="/images/bullet3sel.gif" width="9" height="9" vspace="15"/>

<rb:script>
  // The following blocks contains the processing of a body tag
  // and enclosed chapter and section tags. 
  //
  var rb$pageTitle;
  var rb$rqdSect = rb.page.formdata() ? rb.page.formdata().rbsect : null;
  var rb$defSect;
  var rb$tagsSect;
  var rb$tagsMenu;

  function rbwritemenubulletstyle()
  {
    rb$menuBullets.style = true;  // use style instead of image.
    if (rb$menuBullets.length>0)
      with (rb.page)
      {
        write('<style type="text/css">\n');
        for (var i=0; i<rb$menuBullets.length; i++)
        {
          var n, s, g = new String;
          var b = rb$menuBullets[i];
          for (var x in b)
            switch(x)
            {
              case "normal" : n = 'background: url("'+b.normal+'") no-repeat; '; break;
              case "select" : s = 'background: url("'+b.select+'") no-repeat; '; break;
              case "width"  :
              case "height" : g += x+': '+parseInt(b[x])+'px; '; break;
              default       : break;
            }
          if (n)
            write(' .rbmenubullet'+b.level+'{'+g+n+'}\n');
          if (s)
            write(' .rbmenubullet'+b.level+'sel{'+g+s+'}\n');
        }
        write('</style>\n');
      }
  }


</rb:script>

