// File that contains rendering information to translate glossary anchor tags into 
// client side references.
function txlateGlossary(htmlText) {
  rb.debug.write("Parsing:<br>"+Util.doXmp(htmlText));
  var aParser = new XmlParser;
  aParser.mode(XmlModeHTML);
  if (aParser.build(htmlText)) {
     // Find all anchor tags with term attribs and translate them.
    var tags = aParser.root.select('//(a | A)[@term]');
    rb.debug.write("Anchor list contains "+tags.length+" elements<br>\n");
    for (var i=0;i<tags.length;i++) {
       rb.debug.write("Doing replace...<br>\n");
       tags[i].attributes.href="/glossary/quickview.rhtm?term="+tags[i].attributes.term;
       tags[i].attributes.onClick="window.open('/glossary/quickview.rhtm?term="+tags[i].attributes.term+"','glossary','resizable=yes,width=780,height=450,scrollbars');return false";
    }
    var res=aParser.root.execute();
  }
  else {
    var res="Bad HTML:<br>\n";
    for (var i=0;i<aParser.errors.length;i++) {
      res += Util.doXmp(aParser.errors[i])+'<br>\n';
    }
  }
  return res;
}

var Glossary = {}

Glossary.txlate = txlateGlossary;

Glossary.validateSeeAlso = function(seealso) {
  var failedSep = '';
  var failedList = '';

  if (seealso.length>0) {
    var alsoList = (seealso).split(/\s*,\s*/);
    dumpObj(alsoList,"alsoList");
    for (var i=0;i<alsoList.length;i++) {
      // Does this term exist in the glossary?
      var it=rb.catalogue.item.get({ID:SysData.RootCats.Glossary, idMkr:"N", allLevels:0, keyword:[alsoList[i]], searchInCode:true});
      rb.debug.write("Checking seeAlso["+alsoList[i]+"]...");
      if (!it.getNextRow()) {
        rb.debug.write("Failed");
        failedList += (failedSep + alsoList[i]);
        failedSep = ', ';
      }
      rb.debug.write("<br>\n");
    }
  }
  rb.debug.write("Returning failed list containing ["+failedList+"]<br>\n");
  return failedList;
}

Glossary.createEntry = function(params, key, overwrite) {
var done = false;

  dumpObj(params, "createEntrt.params");

  if (params.term==null || params.term=='')
    return {msg:"Missing glossary term name"};

  if ((params.term).search(/[^\w\s]/)>=0)
    // Term contains illegal characters. 
    return {msg:"Glossary term contains illegal characters."};

  if (params.body==null || params.body.length==0)
    return {msg:"There must be a textual definition of the term"};
  else {
    // Validate the 'seealso' list.
    var failedSeeAlso = Glossary.validateSeeAlso(params.seealso);
    if (failedSeeAlso.length>0) {
      return {seealso:true, msg:"The following 'see also' terms are not in the glossary:"+failedSeeAlso};
    }
  }
  // We may be creating or editing an existing term. If so the 'key' parameter let's us locate the old entry.
  if (key) {
    // Key specified - try to read the record.
    var it = rb.catalogue.item.get({ID:SysData.RootCats.Glossary, idMkr:"N", allLevels:0, keyword:[key], searchInCode:true});
    if (it.getNextRow()) {
      // OK - existing glossary entry. In this case we *must* have a change 'reason'
      if (params.reason==null || params.reason.length==0) {
        rb.debug.write("NO CHANGE REASON PROVIDED<br>\n");
        return {msg:"Must provide a change history comment"};
      }
      else {
        if (it.stockData==null)
           it.stockData = {};
        if (it.stockData.seealso==null)
           it.stockData.seealso='';
        if (it.stockData.history==null)
           it.stockData.history = [];

        // Have an existing entry. Only save if there is a change.
        var needSave = (it.itemData    != params.body)
                  ||   (it.description !=  params.term)
                  ||   (it.code != (term).toUpperCase())
                  ||   (it.stockData.seealso != params.seealso);
        if (needSave) {
          it.itemData    =  params.body;
          it.description =  params.term;
          it.code        = (params.term).toUpperCase();
          it.stockData.seealso = params.seealso;
          it.stockData.history.unshift({reason:params.reason, uname:extauth.sessionUser, date:new Date()})
          rb.catalogue.item.modify(it);
          return {change:'updated'}
        }
        return {change:'none'};
      }
    }
  }
  // Trying to create a new entry. See whether the term already exists and prevent overwrite.
  var res = {change:'created'};

  var it = rb.catalogue.item.get({ID:SysData.RootCats.Glossary, idMkr:"N", allLevels:0, keyword:[(params.term).toUpperCase()], searchInCode:true});
  if (it.getNextRow()) {
    if (overwrite) {
      // Overwrite this entry.
      it.itemData    =  params.body;
      it.description =  params.term;
      it.code        = (params.term).toUpperCase();
      if (it.stockData==null)
        it.stockData = {history:[]};
      it.stockData.seealso = params.seealso;
      it.stockData.history.unshift({reason:params.reason, uname:extauth.sessionUser, date:new Date()})
      rb.catalogue.item.modify(it);
      return {change:'updated'}
    }
    return {msg:'Attempt to overwrite existing entry'};
  }
  // Only remaining option is to create a new entry.
  var id = rb.catalogue.item.create(
                       {code:(params.term).toUpperCase(), 
                        description:params.term, itemData:params.body,
                        stockData:{history:[],seealso:''}});
  if (id>=0) {
    rb.catalogue.category.linkItem(id, SysData.RootCats.Glossary);
    return {change:'created'}
  }
  return {msg:"Failed to create new entry - catalogue failure"};
}

Glossary.remove = function(term) {
  var it = rb.catalogue.item.get({ID:SysData.RootCats.Glossary, idMkr:"N", allLevels:0, keyword:[(term).toUpperCase()], searchInCode:true});
  if (it.getNextRow()) {
    // Term exists - remove now.
    return rb.catalogue.item.remove(it.itemID)>=0;
  }
  return false;
}

Glossary.importXmlData = function(importdata, forceDelete) {
  var aParser = new XmlParser;
  var stats   = {fails:0, create:0, update:0, nochange:0, remove:0, errs:[]};
  
  if (aParser.build(importdata)==null)
    // File contains XML structural errors - display errors and exit.
    return aParser.errors;
  
  // Step through each <entry> tag.
  var entries = aParser.root.select('/glossary/entry[@term and string!=""]');
  var done = false;
  var lastFailCount = 0;
  
  while (!done) {
    var failEntries = {tags:[], errs:[]};
    
    for (var i=0;i<entries.length;i++) {
      var thisEntry = entries[i];
      
      // Must have a 'name' attribute and the body must *not* be empty.
      rb.debug.write("<p>Got glossary entry for <b>"+entries[i].attributes.term+"</b><br>"+entries[i].execute().bodytext()+"</p>\n");
      
      if (thisEntry.attributes.remove=='yes' || forceDelete) {
        if (Glossary.remove(thisEntry.attributes.term))
          stats.remove++;
      }
      else {
        var seealso = thisEntry.attributes.seealso;
        if (seealso==null)
          seealso = '';
        var res = Glossary.createEntry({term:thisEntry.attributes.term, body:thisEntry.execute().bodytext(), seealso:seealso, reason:'XML Import'},null,true);
        if (res.msg!=null) {
          // If the creation failed simply because of seealso errors then add to the 'error' list.
          if (res.seealso) {
            failEntries.tags.push(thisEntry);
            failEntries.errs.push("Failed to import '"+thisEntry.attributes.term+"' - "+res.msg+"<br>\n");
          }
          else {
            stats.errs.push("Failed to import '"+thisEntry.attributes.term+"' - "+res.msg+"<br>\n");
            stats.fails++;
          }
        }
        else {
          switch (res.change) {
          case 'updated':
            stats.update++;
            break;
          case 'none':
            stats.nochange++;
            break;
          case 'created':
            stats.create++;
            break;
          }
        }
      }
    }
    done = true;
    // If we get to this point and there are entries in the 'failEntries' array - AND there are less errors this time - try again.
    if (failEntries.tags.length>0) {
      if (lastFailCount==0 || failEntries.tags.length<lastFailCount) {
        // OK - may be 'seealso' reference errors. Reparse the fail nodes.
        entries = failEntries.tags;
        lastFailCount = failEntries.tags.length;
        done = false;
      }
      else {
        // Same number of errors as last time so we need to give in - there are real unresolved seealso entries.
        stats.errs = stats.errs.concat(failEntries.errs);
      }
    }
  }
  return stats;
}
