/*********************************************************
Component:    Redbourne Event Management Framework.
Filename:     member.js
Author:       Peter Wilson
**********************************************************
Overview
========
This file contains the basic definitions of the core
MemberShip services framework. Included in the file are
the following elements:
   Member   -  class that encapsulates and maps information
               between a member and the underlying 'Contact'.
   MemberIt -  Derived class that implements an iterator.
	BasicMemberApplet -
               Provides the core set of member attributes.
**********************************************************
$Id: member.js,v 1.1 2002/05/15 11:56:04 petew Exp $
**********************************************************
$Log: member.js,v $
Revision 1.1  2002/05/15 11:56:04  petew
Initial CVS Checkin.

Revision 1.2  2002/02/27 16:48:00  petew
Version 1.1 of the site - includes the HTML editor interface along with
the new style proposed by XPRL.

Revision 1.1  2002/02/14 07:49:48  petew
Inital XPRL checkin - nothing finished yet!

Revision 1.1  2001/08/29 15:39:37  rob
*** empty log message ***

Revision 1.10  2001/08/14 17:01:25  steved
Added an extended property of "phone2"

Revision 1.9  2001/07/31 12:35:46  steved
Catch some additional cases for building of contact "name"

Revision 1.8  2001/07/25 11:40:30  steved
Attempt to fix the "User create loses meta-data" bug.

Revision 1.7  2001/07/20 18:00:50  rob
Import some fixes from ISOC implementation:
  Remove existing properties all applet copy functions
   (bug manifested itself as iterator property leakage)

Revision 1.6  2001/07/19 13:03:25  rob
Member interface changes post ISOC implementation
Added event management membership class

Revision 1.1  2001/07/17 19:28:03  rob
new implementation

Revision 1.4  2001/07/12 07:24:52  rob
Added understanding of date objects.

Revision 1.3  2001/05/24 09:00:02  steved
Fix a mismatched brace

Revision 1.2  2001/05/04 15:09:30  petew
Some minor bug fixes.

**********************************************************
Copyright (c) Redbourne Inc, 2001
**********************************************************/

/********************************************************************************
Member
Constructor for the JavaScript 'Member' class. This takes an optional
'input_data' parameter that contains some initial data for the member. IF
PRESENT the method tries to initialise the object as follows:

    Attempt to locate a contact with the ID or uname in input_data
    Copy the input_data methods over the top of the resulting object.

********************************************************************************/
function Member(input_data) {

   // Create attributes used to manage meta-data. The basic
   // aim here is to make the member structure look the same as
   // the basic contact record - and hide the complexity - so
   // the original 'customData' field is still used by Pattern
   // Pages - but the field past to the contact template 
   // is the 'meta' field.
   this.rb$meta             = new Object;         // To store meta-data.
   this.rb$meta.userData    = new Object;         // To store user meta-data.
   this.customData          = this.rb$meta.userData;
   this.rb$meta.rb$isMember = true;

	if (input_data != null) {
      if (input_data.ID != null && input_data.ID != "")
			 this.get(input_data.ID);
      else if (input_data.uName!=null)
			 this.get(-1, input_data.uName);

		// Now overlay the properties from the 'input_data' object.
		this.overlayProperties(input_data);
	}
}
/********************************************************************************
'Member' Class Methods.
These are created as functions then referenced from the Member prototype.
 ********************************************************************************/

/*****************************************************************
Member.copyProperties
Utility method that takes a list of property values and copies
them from 'this' input data object to the target object.
*****************************************************************/
Member.copyProperties = function (sourceData, propList, useThisTarget) {
    var prop;

    for (prop in propList){
	if(sourceData[propList[prop]] != null) {
	    useThisTarget[propList[prop]] = sourceData[propList[prop]];
	}
    }
}

/*****************************************************************
Member.removeProperties
Utility method that takes a list of property names and removes
those properties from the target object.
*****************************************************************/
Member.removeProperties = function (propList, useThisTarget) {
var prop;

    for (prop in propList){
		  // Remove it.
		  // rb.debug.write('<p>Removing property "'+
        if (useThisTarget[propList[prop]]!=null)
		     delete useThisTarget[propList[prop]];
		  //useThisTarget[prop] = null;
    }
}

/*****************************************************************
Member.prototype.overlayProperties
Utility method that takes a list of property values and copies
them from 'this' input data object to the target object.
*****************************************************************/
Member.prototype.overlayProperties = function (sourceData) {
	 // This is easy - simply get each application to overlay 
	 this.callApplet("overlayProperties", sourceData);
}

/*****************************************************************
Member.prototype.callApplet
This is a private method. It calls the named method on each of the
registered 'applets'. If the applet wants to do some work then
this is how it is achieved.

The function - if present - receives the parameters passed to this
method.
*****************************************************************/
Member.prototype.callApplet = function (method_name) {
    var i;

    for (i in Member.applets) {
	if (typeof Member.applets[i]=="object") {
	    // If it has the named method then call it.
	    if (Member.applets[i][method_name]) {
		Member.applets[i][method_name].apply(this, arguments);
	    }
	}
    }
}

/*****************************************************************
Member.prototype.makeContact - private
This method takes a 'member' and builds the corresponding 'contact'
structure - ready to used in the contact template API. This is not
intended to be a public interface!
*****************************************************************/
Member.prototype.makeContact = function () {
   // Create the real contact record by copying fields from this object.
   var newContact = new Object;

	// Deal with our enhanced data-members.
	this.callApplet("makeContact", newContact);
	
   // The applets may have set an error condition - if so then DON'T 
	// return a contact record.
	if (this.rb$error)
		return;

   // Make sure we reference the basic custom data.
   delete this.rb$meta.userData;
	this.rb$meta.userData          = this.customData;
   newContact.customData          = this.rb$meta;
   newContact.customData.userData = this.customData;

	return newContact;
}

/*****************************************************************
Member.isMember
Check the contact information to see whether this is a member.
If it is _then_ return TRUE, otherwise return FALSE.
*****************************************************************/
Member.isMember = function (aContact) {
	 return (aContact.customData!=null && (aContact.customData.rb$isMember==true));
}
/*****************************************************************
Member.prototype.convertContactToMember - private
Given a raw (old) contact - convert it to a member record.
*****************************************************************/
Member.prototype.convertContactToMember = function (aContact) {

    this.rb$meta             = new Object;
    this.rb$meta.userData    = aContact.customData;
	if( this.rb$meta.userData == null )
		this.rb$meta.userData = new Object;
	this.customData = this.rb$meta.userData;
    this.rb$meta.rb$isMember = true;

	// Now - get the applets to do any conversion they require.
	this.callApplet("convertContactToMember", aContact);
}

/*****************************************************************
Member.prototype.makeMember - private
This member takes a 'contact' as defined by the contact template
and initialises the fields in this member object. This is an
internal method - not inteded for public use.
*****************************************************************/
Member.prototype.makeMember = function (aContact) {

	// Modify the meta-data storage for this object. This bit of magic
	// allows the application developer to use the member in exactly 
	// the same way as a 'contact' but behind the scenes ends up sharing
	// the meta-data with other applications.
	// 1. Move contact.customData to member.rb$meta. 
	// 2. 'member.customData' referenced from 'member.rb$meta.userData'.
	 if ( !Member.isMember(aContact) ) {
			// NOT originally a member!
			this.convertContactToMember(aContact);
	 }
    else {
		  // A member.
		  this.rb$meta    = aContact.customData;
		  this.customData = aContact.customData.userData;

        // Now give each applet the opportunity to extract their own information.
        // Deal with our enhanced data-members.
        this.callApplet("makeMember", aContact);

        this.customData = this.rb$meta.userData;
	 }
}

/********************************************************************************
Member.prototype.create
Use the data in the membership object to create a new contact record to represent
the new member. The function returns -1 if it fails.
********************************************************************************/
Member.prototype.create = function () {

   // Create the real contact record by copying fields from this object.
   var newContact = this.makeContact();
   this.ID        = -1;

   if (newContact) {
      this.ID = rb.contact.createIndividual(newContact);
      if (this.ID>0) {
         // Created it OK. Read it back to make sure all the fields are correctly updated.
        this.get();
      }
   }

   return this.ID;
}

/********************************************************************************
Member.prototype.save
Save the contents of the membership record to the contact template. This record
must already exist. 

This function exists as aMember.save();
********************************************************************************/
Member.prototype.save = function () {
var newContact = this.makeContact();
var res        = -1;

   if (newContact) {
		 res = rb.contact.modifyIndividual(newContact);
	}
   return res;
}

/********************************************************************************
Member.prototype.serialiseHeader
Get member field names into an array (e.g. for use in CSV file output).
********************************************************************************/
Member.prototype.serialiseHeader = function () {
    var res = new Array();
    this.callApplet("serialiseHeader", res);
    
    return(res);
}

/********************************************************************************
Member.prototype.serialiseProperties
Get member field names into an array (e.g. for use in CSV file output).
********************************************************************************/
Member.prototype.serialiseProperties = function (columns) {
    var res = new Array();
    var flat_res = new Array();
    this.callApplet("serialiseProperties", columns, this, res);
    for(var a in res){
	// rb.debug.write("res["+a+"] = "+res[a].toSource()+"<BR>\n");
	flat_res = flat_res.concat(res[a]);
    }
    // rb.debug.write("flat_res = "+flat_res.toSource()+"<BR>\n");
    
    return(flat_res);

}

/********************************************************************************
Member.prototype.remove
Remove the member from the contact record.
********************************************************************************/
Member.prototype.remove = function () {
   return rb.contact.remove(this.ID);
}

/********************************************************************************
Member.prototype.get
Method in the Member component that populates the Member record from the specified
contact ID, or uname. There are two ways of calling this function. The first
takes an id or uname, the second takes no parameters and tries to use the contents
of the object.

This function exists as aMember.get();
********************************************************************************/
Member.prototype.get = function (id, uname) {
var aContact;
var id 
   if (arguments.length==0)
      aContact = rb.contact.individualInfo(this.ID, this.uName);
   else
      aContact = rb.contact.individualInfo(id, uname);

   if (aContact!=null) {
      this.ID = aContact.ID;
      this.makeMember(aContact);
   }
}

Member.applets = new Array;
/********************************************************************************
MemberIt - Member Iterator.
This is a new JavaScript class - it isn't directly instantiated by the application
instead being returned from the Member.get class method. The class 'looks' exactly
like an iterator - but behind the scenes does the manipulation to extract contact
info into the member structure.

Because an iterator is basically a subclass of the basic data structure - with the
addition of the iterator methods. In JavaScript we achieve this simply by setting
the prototype to be an instance of Member.
 ********************************************************************************/
/********************************************************************************
MemberIt
Iterator for member information. This is the constructor. Doesn't do anything 
except at the moment.
********************************************************************************/
function MemberIt(contactIterator) {
   this.rb$Iterator = contactIterator;
}

/********************************************************************************
* Force the iterator to be a subclass of the Member.
*********************************************************************************/
MemberIt.prototype = new Member;

/********************************************************************************
MemberIt.prototype.getNextRow
Overload for the iterator getNextRow method.
********************************************************************************/
MemberIt.prototype.getNextRow = function () {
var res;

   do {
      res = this.rb$Iterator.getNextRow();
   } while (res && !Member.isMember(this.rb$Iterator))

   // Copy the data from the iterator to 'this'.
   if (res) {
		 // Load the member fields from the iterator.
		this.makeMember(this.rb$Iterator);
	}
   return res;
}
/********************************************************************************
MemberIt.prototype.setNextRow
Overload for the iterator setNextRow method. Basically a NOP.
********************************************************************************/
MemberIt.prototype.setNextRow = function (index) {
   return this.rb$Iterator.setNextRow(index);
}
/********************************************************************************
MemberIt.prototype.numberOfRows
Return the number of rows - returns the iterator value.
********************************************************************************/
MemberIt.prototype.numberOfRows = function (index) {
   return this.rb$Iterator.numberOfRows();
}
/********************************************************************************
MemberIt.prototype.currentRow
Return the number of rows - returns the iterator value.
********************************************************************************/
MemberIt.prototype.currentRow = function (index) {
   return this.rb$Iterator.currentRow();
}
/********************************************************************************
Member.get
Create an iterator for members.
********************************************************************************/
Member.get = function (keywords, flags) {
  var iterator;
  var memIt;
  
  // Force the correct set of flags for finding individuals.
  if (flags==null)
    flags = new Object;
  
  flags.searchIndividual = true;
	 flags.searchOU         = false;
   flags.searchCommunity  = false;
   flags.searchName       = true;
   flags.sort             = 'name';
   
   iterator = rb.contact.get(keywords, flags);
   memIt    = new MemberIt(iterator);
   return memIt;
}


/********************************************************************
rb$basicMember
Define basic applet extension for a member - this defines the 
basic information set that is common to all members. 
*********************************************************************/
var rb$basicMember = new Object;
rb$basicMember.props = [ 
		  // Standard 'contact' properties.
        "ID",
        "phone",
        "uName", 
        "name",
        "fax", 
        "email",
        "keywords",
        "parent"
];
rb$basicMember.ext_props = [ 
       // Extended set of properties specific to a member.
		  "title",
		  "first_name",
		  "initial",
		  "surname",
      "addr1", 
      "addr2",
      "addr3",
      "addr4",
		  "organisation",
		  "membership_number",
		  "gender",
      "post_code",
      "phone2",
      "mobile",
      "country",
      "description",
		  "membership_class"
];
rb$basicMember.ext_date_props = [ 
    // Extended set of properties specific to a member which are
    // date rather than the default text objects
		"dob",
		"join_date"
];

/********************************************************************************
rb$mem$basic$makeContact
Initialise the fields of the Contact data structure from the member object.
********************************************************************************/
rb$basicMember.makeContact = function (Method, Contact) {

  // Copy the core contact properties.
  // Copy the basic contact fields to the contact record.
  Member.removeProperties(rb$basicMember.props, Contact);
  Member.copyProperties(this, rb$basicMember.props, Contact);

  // Manually generate the contact 'name' field by concatenating together the component elements.
  Contact.name = "";
  if (this.surname!=null && this.surname!='')
     Contact.name += this.surname;

  if (this.first_name!=null && this.first_name!='') {
     if (Contact.name.length>0)
        Contact.name += ', ';

     Contact.name += this.first_name+' ';
  }

  if (this.initial!=null && this.initial!='')
     Contact.name += this.initial+' ';

  Contact.name = Contact.name.replace(/^\s*$/, this.name);

  if (this.keywords) {
		// Keywords should be an array or strings. These are going to be
		// serialised into a single string and put in the description field.
		// First of all - is this an array?
		if ((this.keywords instanceof String) || (typeof this.keywords=='string')) {
			 this.keywords = this.keywords.split(/\s*,\s*/);
		}
      else if (!(this.keywords instanceof Array)) {
			 rb.debug.error("Member.keywords attribute should be an array or string of keywords!");
			 return;
		}
		// Validate the string. Should ONLY contain a list of alpha-numeric names, separated by spaces.
		var str = this.keywords.join(' ');

      // Make sure the keyword list ONLY contains alpha characters.
		if (str.search(/[^a-zA-Z0-9\s\-]/)>=0) {
			 // Error - the keyword list is contrained to only contain alpha-numerics.
			 this.rb$error = "Illegal characters in keyword list.";
			 return;
		}
		Contact.description = this.keywords.toSource();
  }


  // The extension fields are copied into the meta-data.
  this.rb$meta.core = new Object;
  Member.removeProperties(rb$basicMember.ext_props, this.rb$meta.core);
  Member.removeProperties(rb$basicMember.ext_date_props, this.rb$meta.core);
  Member.copyProperties(this, rb$basicMember.ext_props, this.rb$meta.core);
  Member.copyProperties(this, rb$basicMember.ext_date_props, this.rb$meta.core);
}

/********************************************************************************
rb$mem$basic$convertContactToMember
Convert raw contact data to a form that can be used by the member subsystem.
This involves doing some analysis of some of the core fiels and moving the
resulting data around somewhat.
********************************************************************************/
rb$basicMember.convertContactToMember = function (Method, aContact) {
  
	 // rb.debug.write("<h3>Called rb$basicMember.convertContactToMember</h3>\n");
  if (this.rb$meta.core==null)
    this.rb$meta.core = new Object;
  
  for (var x in aContact) {
    this[x] = aContact[x];
  }
  if (this.description!=null) {
    delete this.description;
    this.description = null;
  }
  
	 // 'description' has been hijacked to support keyword searching. Need to
	 // move the original description into the meta-data field.
	 if (aContact.description!=null) {
     // Move it.
     this.rb$meta.core.description = aContact.description;
   }
   // Process the name field a little.
   if (aContact.name) {
     // Need to take this and break it into the various parts. Break into words.
     var fullName   = new String(aContact.name);
     var nameTokens = fullName.split(/\s+/);
     
     if (nameTokens.length>0) {
       // Is there a title at the begining?
       var key = nameTokens[0].toLowerCase();
       if ({"mr":true, "mrs":true, "ms":true, "miss":true, "dr":true}[key]!=null) {
         // rb.debug.write("<p>p1="+nameTokens[0]+'</p>\n');
         this.title = nameTokens[0];
         nameTokens.shift();
       }
     }
     if (nameTokens.length>1) {
       // Next one should be the first name.
       this.first_name = nameTokens[0];
       // rb.debug.write("<p>p2="+nameTokens[0]+'</p>\n');
     }
     
     if (nameTokens.length>2) {
       // This one is the initial.
       this.initial = nameTokens[1][0];
     }
     
     // Get the last name.
     if (nameTokens.length>=1) {
       this.surname = nameTokens.pop();
     }
     
     // Now regenerate the name.
     this.name = "";
     var some=false;
     if (this.surname) {
       this.name += this.surname;
       some = true;
     }

     if (this.first_name) {
       if (some)
          this.name += ', ';

       this.name += this.first_name+' ';
     }     
     if (this.initial)
       this.name += this.initial+' ';
     
     // Create a convenient 'shortName' of first name and last name.
     this.shortName = "";
     if (this.first_name)
       this.shortName += this.first_name+' ';
     
     if (this.surname)
       this.shortName += this.surname;

   }
}
/********************************************************************************
rb$mem$basic$makeMember
Thie 'makeMember' handler for the basic info set. This copies all the top level 
properties of the contact to the member object. It also takes all the properties
of the 'core' metadata field and copies those to the top level - then deletes the
top level version.
 ********************************************************************************/
rb$basicMember.makeMember = function (Method, aContact) {
  
  // Basic 'contact' top level data set.
  for (var x in aContact) {
    this[x] = aContact[x];
  }
  
  // Now the enhanced set of properties.
  //	 this.rb$meta.core = new Object;
  if (this.rb$meta.core){
    Member.removeProperties(rb$basicMember.ext_props, this);
    Member.removeProperties(rb$basicMember.ext_date_props, this);
    Member.copyProperties(this.rb$meta.core, rb$basicMember.ext_props, this);
    Member.copyProperties(this.rb$meta.core, rb$basicMember.ext_date_props, this);
  }
  
  // Recontruct the 'keywords' array from the contact description field.
  if (aContact.description != null && (aContact.description.length > 0)) {
    this.keywords = eval(aContact.description);
  }
  // Build the shortName field.
  this.shortName = "";
  if (this.first_name)
   this.shortName += this.first_name+' ';

  if (this.surname)
   this.shortName += this.surname;
}

/********************************************************************************
rb$mem$basic$overlayProperties
This method is responsible for copying data from an external object into the
Member object. In the case of this basic set - deal with the core set of 
contact data fields as well as the basic extended set.
 ********************************************************************************/
rb$basicMember.overlayProperties = function (name, data) {

	 // Copy the core contact properties.
    Member.copyProperties(data, rb$basicMember.props,     this);
    Member.copyProperties(data, rb$basicMember.ext_props, this);
    var dateob = new Object();
    for(var d in rb$basicMember.ext_date_props){
	if(data[rb$basicMember.ext_date_props[d]] != null)
	    dateob[rb$basicMember.ext_date_props[d]] = Util.parseDate(data[rb$basicMember.ext_date_props[d]]);
    }
    Member.copyProperties(dateob, rb$basicMember.ext_date_props, this);
	
}

rb$basicMember.serialiseHeader = function (Method, headers) {
    var header =  [].concat(rb$basicMember.props, rb$basicMember.ext_props, rb$basicMember.ext_date_props);
    // rb.debug.write("into rb$basicMember.serialiseHeader with ret = "+headers.toSource()+"<BR>");
    headers.push(header);
    // rb.debug.write("left rb$basicMember.serialiseHeader with ret = "+headers.toSource()+"<BR>");

    return;
}
rb$basicMember.serialiseProperties = function (Method, cols, data, res) {
    var header =  [].concat(rb$basicMember.props, rb$basicMember.ext_props, rb$basicMember.ext_date_props);
    var myHeader = new Object();
    for(i in header){
	myHeader[header[i]] = true;
    }
    if(cols == null){
	cols = header;
    }
    for(var prop = 0; prop < cols.length; prop++){
	if(myHeader[cols[prop]]){
	    var value = ""
	    if(this[cols[prop]] != null){
		if(typeof this[cols[prop]] == "object" && this[cols[prop]].constructor == Date)
		    value = Util.fmtShortDate(this[cols[prop]]);
		else
		value = this[cols[prop]];
		if(typeof value != "string")
		    values = values.toString();
	    }
	    res.push(value);
	}
    }
}
    
    

Member.applets.basicApplet = rb$basicMember;













