/*******************
NewsItem
********************
Wrapper class for news articles. Each news article is a catalogue item. If the
length of the article body is <NewsItem.maxItemLength characters it is stored in the stockData field - 
otherwise it is in a separate file.
********************/
// function NewsItem(input_data) {Doc.call(this,input_data);}
function NewsItem(input_data, andContent) {
var restored = false;
var it = null;

   this.customData = {};
   this.privData   = {};
   this.mime       = 'text/html';
   this.state      = 'Draft';
   this.ID         = -1;
   this.category   = -1;         // where it should eventually reside (as well as root).
   this.fileID     = -1;         // Where to find the content for this document.
   this.created    = new Date();
   this.rb$type    = this.docType;

   if (input_data == null)
      input_data = {};
   
   this.writeAccess  = '';        // Will default to the author before saving.
   this.readAccess   = '';        // this is the default - and means everyone.

   var id       = (input_data.ID == null)?-1:input_data.ID;
   var authorID = (input_data.authorID==null)?-1:input_data.authorID;

   // rb.debug.write("<p>CatID="+id+", authorID="+authorID+"<br>");
   
   // If there is an ID then try to read this catalogue item.
   if (id>0)
      var it = rb.catalogue.item.get({ID:id, idMkr:'I', allLevels:false});
   else if (authorID>0)
      // Might be under the author - a contact template ID.
      var it = rb.catalogue.item.get({ID:authorID, idMkr:"C", allLevels:false});

   if (it!=null) {
      if (it.getNextRow() && NewsItem.isInstance(it)) {
         restored = true;

         // If the content is required then read it now.
         this.makeDoc(it);

         if (andContent==null || andContent==true)
            this.getContent();
          for (var p=0;p<Doc.props2.length;p++) {
             if (it[Doc.props2[p].item]!=null)
                this[Doc.props2[p].news] = it[Doc.props2[p].item];
          }
      }
   }
   // Overlay the rb$params parts.
   for (var p in Doc.props3) {
     if (input_data[p])
        this[p] = input_data[p];
   }
   // Deal with assocParent array.
   if (input_data.assocParents!=null) {
     if (typeof input_data.assocParents == 'object')
       this.assocParents = input_data.assocParents;
     else
       this.assocParents = [input_data.assocParents];
   }
}

// ******* getCategories
// Method to return set of all specific categories.
NewsItem.getCategories = function() {
   var catList = [];
   var it = rb.catalogue.category.get({ID:SysData.RootCats.News, allLevels:1, idMkr:'N', sortByName:true});

   if (it) {
     while (it.getNextRow()) {
        catList.push({ID:it.nodeID, name:it.name, description:it.description, hasChildren:(it.contentsMkr!=null && it.contentsMkr!='')});
     }
   }
   return catList;
}
NewsItem.prototype = new Doc;
NewsItem.prototype.docType = Doc.prototype.docType+'/news';

/*****************************************************************
Doc.isInstance
*****************************************************************/
NewsItem.isInstance = function (catItem) {
   return (catItem.itemData != null && catItem.itemData.rb$type!=null && ((catItem.itemData.rb$type).indexOf(NewsItem.prototype.docType)==0));
}

/********************************************************************************
NewsItem.prototype.create
Use the data in the Doc object to create a new category item to represent
the new article. The function returns -1 if it fails. The method checks the size 
of the content and write the content to *either* a file or the stock data area.
********************************************************************************/
NewsItem.prototype.create = function () {
var item;
var draft = (this.state=='Draft');

   // First of all - create the generic document.
   if (!draft && SysData.RootCats.News!=null) {
      // Publishing so link to the root of the tree.
      if (this.parent==null)
         this.parent = SysData.RootCats.News;
   }
   var ID = this.ID = Doc.prototype.create.call(this);

   return this.ID;
}

/********************************************************************************
NewsItem.prototype.save
Save a modified version of this news item. if the generic save of the doc works
check to see whether we're publishing for the first time. If we *are* publishing
then we need to attach to the correct news node. The news node is stored in the
'parentCategory' attribute. If there isn't one then save under the generic
news category.
********************************************************************************/
NewsItem.prototype.save = function () {
var res     = false;
var publish = (this.state!='Draft');

  if (this.parent==null)
    this.parent = SysData.RootCats.News;

  return (Doc.prototype.save.call(this));
}

/********************************************************************************
NewsItemIt - NewsItem Iterator.
This is a new JavaScript class - it isn't directly instantiated by the application
instead being returned from the Event.get class method. The class 'looks' exactly
like an iterator - but behind the scenes does the manipulation to extract contact
info into the member structure.

Because an iterator is basically a subclass of the basic data structure - with the
addition of the iterator methods. In JavaScript we achieve this simply by setting
the prototype to be an instance of Event.
 ********************************************************************************/
/********************************************************************************
NewsItemIt
Iterator for NewsItems information. This is the constructor. Doesn't do anything 
except at the moment.
********************************************************************************/
function NewsItemIt(catIterator) {
    this.rb$Iterator = catIterator;
}

/********************************************************************************
* Force the iterator to be a subclass of the Event.
*********************************************************************************/
NewsItemIt.prototype = new NewsItem;

/********************************************************************************
NewsItemIt.prototype.getNextRow
Overload for the iterator getNextRow method.
********************************************************************************/
NewsItemIt.prototype.getNextRow = function (wantWrite) {
var res = false;
var test = (wantWrite ? 'write' : 'read');

  do {
    do {
      res = this.rb$Iterator.getNextRow();
    } while (res && !NewsItem.isInstance(this.rb$Iterator));
    // Copy the data from the iterator to 'this'.
    if (res) {
      // Load the member fields from the iterator.
      this.makeDoc(this.rb$Iterator);
      var stattest = (this.rb$status==null || (this.rb$status).indexOf(this.state)>=0);
    }
  } while ((res==true && ((stattest==false) || (this.checkAccess('super,newsadmin')[test]==false))));
  return res;
}
/********************************************************************************
NewsItem.get
Create an iterator for NewsItems.
********************************************************************************/
NewsItem.get = function (data) {
var iterator;
var newsIt;

   if (data==null)
      data = {ID:SysData.RootCats.News, idMkr:'N', allLevels:1};

   iterator = rb.catalogue.item.get(data);
   newsIt = new NewsItemIt(iterator);
   newsIt.rb$status = data.status;
   return newsIt;
}


