/***************************
select.js
This file encapsulates the basic functions required to maintain
a list of selected users in the session data. Each entry in
the selection list is represented as a {id,uname} pair.
***************************/
Selection = new Object;

Selection.data;
Selection.Empty = {url:'',cache:{length:0},users:{}, client:{}, switches:[], flags:{}};

/**************
Completely reset the selection data ready for something new. Deletes any information
laying around in the sesion data between executions of the selection list.
**************/
Selection.reset = function() {
  var sessionData = rb.page.session.read(sessionData);

  Selection.data =
  sessionData.selection = Selection.Empty;
  Selection.clear();

  rb.page.session.write(sessionData);
}

/**************
Start - going to use the selection data now - all config changes
MUST have been made bye now!
**************/
Selection.Start = function() {
var selection = Selection.load();

  // MUST be at least ONE selection box (switch) if not then add the default.
if (false) {
  if (Selection.data.switches.length==0) {
    Selection.data.defaultModel = true;
    Selection.addSwitch({name:'<img src="/images/tick.gif">', id:'sel'});
  }
}
  for (var i=0;i<selection.switches.length;i++)  {
    if (selection.users[selection.switches[i].id]==null)
      selection.users[selection.switches[i].id] = {length:0}
  }
 
  if (Selection.data==null) {
    var sessionData = rb.page.session.read(sessionData);

    if (sessionData.selection==null) {
      sessionData.selection = Selection.Empty;

    }
    Selection.data = sessionData.selection;
  }
  return Selection.data;
}

/**************
Load the data. Generally we don't have to worry about this one.
**************/
Selection.load = function() {
  if (Selection.data==null) {
    var sessionData = rb.page.session.read(sessionData);

    if (sessionData.selection==null) {
      sessionData.selection = Selection.Empty;
    }
    Selection.data = sessionData.selection;
  }
  return Selection.data;
}
/**************
If we've loaded the data it may have changed - so save it all back again.
**************/
Selection.save = function() {
  if (Selection.data) {
    // Has been loaded - so can be saved.
    var sessionData = rb.page.session.read(sessionData);

    sessionData.selection = Selection.data;

    // dumpObj(sessionData, "sessionData");
    rb.page.session.write(sessionData);
  }
}

/**************
addSwitch - add a switch name to the switch list. A switch is simply a name/id pair.
**************/
Selection.addSwitch = function(data) {
  if (Selection.data)
    if (data.name != null && data.id!=null)
      Selection.data.switches.push(data);
}


/**************
Store or save the selection data for a client.
**************/
Selection.setClientData = function(data) {
  if (Selection.data)
    Selection.data.clientData = data;
}
Selection.getClientData = function() {
  return (Selection.data) ? Selection.data.clientData : null;
}


/**************
Initialise the selection cache. Used to store the set of names and ID's on the last 
displayed page of the selection tree.
**************/
Selection.clearCache = function() {
  var selection = Selection.load();
  selection.cache = {length:0};
}
/**************
Add entry to the user cache.
**************/
Selection.addCache= function(data) {
  var selection = Selection.load();

  if (data.ID==null || data.ID<0)
    return false;

  if (data.uName==null || data.uName=='')
    return false;

  if (selection.cache[data.ID]==null)
    selection.cache.length++;

  selection.cache[data.ID] = data.uName;
}
Selection.cacheSize = function() {
  return selection = Selection.load().cache.length;
}

/*************
get_uName. Given an ID return the corresponding uName. check the selection cache. If it's not
there then go directly to the catalogue.
*************/
Selection.get_uName = function(id,swtch) {
var selection = Selection.load();

  if (selection.cache[id]==null) {
    // Not in the cache. Add it now.
    var contact = rb.contact.individualInfo(id);
    if (contact==null)
      return null;
    selection.cache[id]=contact.uName;
  }
  return selection.cache[id];
}

/**************
Add a user to the list
**************/
Selection.add = function(data, swtch) {
  var selection = Selection.load();

  if (data.ID==null || data.ID<0)
    return false;

  if (data.uName==null || data.uName=='')
    return false;

  // The entry is valid - is the swtch value?
  if (selection.users[swtch]!=null) {
    // It's valid so add it now.
    if (selection.users[swtch][data.ID]==null)
      selection.users[swtch].length++;

    selection.users[swtch][data.ID] = data.uName;
  }
  return true;
}

/**************
Remove a specific user
**************/
Selection.remove = function(id,swtch) {
var selection = Selection.load();

  if (selection.users[swtch]!=null) {
    if (selection.users[swtch][id]!=null) {
      delete selection.users[swtch][id];
      selection.users[swtch].length--;
    }
  }
}

/**************
Add or remove a user depending on the state of the second parameter.
**************/
Selection.set = function(id,add,swtch) {
var selection = Selection.load();
var uName = Selection.get_uName(id);

  if (uName) {
    if (add) {
      Selection.add({ID:id,uName:uName},swtch);
    }
    else {
      Selection.remove(id);
    }
  }
}


/**************
Return 'true' if member ID is in the selected list.
**************/
Selection.isSelected = function(id,swtch) {
  var selection = Selection.load();

  if (selection.users[swtch]!=null) {
    return (selection.users[swtch][id]!=null);
  }
}

/**************
Remove all data from the selected list.
**************/
Selection.clear = function() {
  var selection = Selection.load();

  selection.users = {};
  for (var i=0;i<selection.switches.length;i++) 
    selection.users[selection.switches[i].id] = {length:0}
  Selection.save();
}

/**************
Return the URL for the parent.
**************/
Selection.url = function(newUrl) {
  var selection = Selection.load();
  return selection.url;
}
Selection.seturl = function(newUrl) {
  var selection = Selection.load();
  if (newUrl!=null) {
    selection.url = newUrl;
  }
  return selection.url;
}

Selection.dump = function() {
  var selection = Selection.load();
  dumpObj(selection, "selection");
}

