// Declare the 'Log' class - must be defined elsewhere in order to work!
var Log;
var ImgProc;
var SysData;

/*******************
WebPage
********************
This is a subclass of 'Doc' that encapsulates webpages. In this instance the 'description' field contains the
webpage 'path'. The page title is stored in the metaxdata field.
********************/
function WebPage(input_data, andContent, wantDraft) {
var restored = false;
var it = null;

   this.customData  = {};
   this.privData    = {};
   this.mime        = 'text/html';
   this.state       = 'Public';
   this.ID          = -1;
   this.category    = -1;         // where it should eventually reside (as well as root).
   
   // Identify which file to use. 'fileIndex' is used as an index into the fileDefs array within a catalogue item.
   // For web pages most accesses are to the draft version - and so we make this the default.
   this.fileID      = -1;         // Where to find the content for this document.
   this.fileIndex   = (wantDraft==null || wantDraft==true)?1:0;

   this.created     = new Date();
   this.rb$type     = this.docType;
   this.skin        = true;
   this.pageTitle   = '';
   this.pageBanner  = '';
   this.customprops = WebPage.customprops;

   // Keep a record of number of times this document is made live - and the date of the last change.
   this.liveCount = 0;

   this.fileRefs    = [];

   if (input_data == null)
      input_data = {};
   
   this.writeAccess  = '';        // Will default to the author before saving.
   this.readAccess   = '';        // this is the default - and means everyone.

   if (Config.defaultPermissions!=null) {
      this.writeAccess  = Config.defaultPermissions.writeAccess;
      this.readAccess   = Config.defaultPermissions.readAccess;
   }

   var id       = (input_data.ID == null)?-1:input_data.ID;
   var authorID = (input_data.authorID==null)?-1:input_data.authorID;

   // If there is an ID then try to read this catalogue item.
   if (id>0)
      var it = rb.catalogue.item.get({ID:id, idMkr:'I', allLevels:false});
   else if (authorID>0)
      // Might be under the author - a contact template ID.
      var it = rb.catalogue.item.get({ID:authorID, idMkr:"C", allLevels:false});

    if (it!=null) {
      if (it.getNextRow() && WebPage.isInstance(it)) {
         restored = true;

         // If the content is required then read it now.
         this.makeDoc(it);

         if (andContent==null || andContent==true)
            this.getContent();
          for (var p=0;p<Doc.props2.length;p++) {
             if (it[Doc.props2[p].item]!=null)
                this[Doc.props2[p].news] = it[Doc.props2[p].item];
          }
      }
   }
   // Overlay the rb$params parts.
   for (var p in Doc.props3) {
     if (input_data[p])
        this[p] = input_data[p];
   }
   // Copy the additional pageTitle, pageBanner and skin parameters if present.
   if (input_data.skin)
     this.skin = input_data.skin;
   
   if (input_data.pageTitle)
     this.pageTitle = input_data.pageTitle;

   if (input_data.pageBanner)
     this.pageBanner = input_data.pageBanner;

   // Deal with assocParent array.
   if (input_data.assocParents!=null) {
     if (typeof input_data.assocParents == 'object')
       this.assocParents = input_data.assocParents;
     else
       this.assocParents = [input_data.assocParents];
   }
}
WebPage.customprops   = { 
  "skin":1,          // Bool. If true - use rbpagemenu skinning - otherwise raw.
  "pageTitle":1,     // Title to display (eg title tag) - draft
  "livePageTitle":1, // Title to display (eg title tag) - live
  "pageBanner":1,    // Banner to be used on the page - draft
  "livePageBanner":1,// Banner to be used on the page - live
  "lastLive":1,      // Title to display (eg title tag) - date of last make-live.
  "liveCount":1      // Number of times made live.
};

WebPage.prototype = new Doc;
WebPage.prototype.docType = Doc.prototype.docType+'/webpage';

/*****************************************************************
WebPage.isInstance
*****************************************************************/
WebPage.isInstance = function (catItem) {
   return (catItem.itemData != null && catItem.itemData.rb$type!=null && ((catItem.itemData.rb$type).indexOf(WebPage.prototype.docType)==0));
}

/********************************************************************************
WebPage.prototype.create
Use the data in the Doc object to create a new category item to represent
the web page. The function returns -1 if it fails. The method checks the size 
of the content and write the content to *either* a file or the stock data area.
********************************************************************************/
WebPage.prototype.create = function () {
var item;
var draft = (this.state=='Draft');

   // First of all - create the generic document.
   rb.debug.write("<h1>In WebPage.prototype.create</h1>");

   var ID = this.ID = Doc.prototype.create.call(this);

   if (ID<0) {
      rb.debug.write("<h1>Failed generic create</h1>");
      return -1;
   }

   // If this is being published - link it under the WebPages path.
   if (!draft && SysData.RootCats.WebPages!=null) {
      // Publishing so link to the root of the tree.
      if (this.parent==null)
         this.parent = SysData.RootCats.WebPages;
      if (!draft)
         this.publish();
   }

   return this.ID;
}

/********************************************************************************
WebPage.prototype.save
Save a modified version of this web page. If the generic save of the doc works
check to see whether we're publishing for the first time. If we *are* publishing
then we need to attach to the correct news node. The news node is stored in the
'parentCategory' attribute. If there isn't one then save under the generic
news category.
********************************************************************************/
WebPage.prototype.save = function () {
var res     = false;
var publish = (this.state!='Draft');

  if (this.parent==null)
     this.parent = SysData.RootCats.WebPages;
  
  return (Doc.prototype.save.call(this));
}

/********************************************************************************
WebPage.prototype.makeLive
Take the draft web-page and 'make it live'. The draft version is in fileRef[1]. To
make this live we make a copy of the file and store it in the fileRef[0] position.
Reuse the current live index if there is one - otherwisse create a new one.

*If* there is no draft version then do nothing.
********************************************************************************/
WebPage.prototype.makeLive = function () {
var liveVersion  = this.fileRefs[0];
var draftVersion = this.fileRefs[1];
var item = this.makeItem();
var res = false;

  if (thisArticle.state!='Draft') {
    // Is there a draft version?
    rb.debug.write("<h4>In 'makeLive' - A</h4>\n");
    if (draftVersion==null || draftVersion.fileID<=0) {
      // NO draft version - exit here!
      rb.debug.write("<h1>1 - no draft version</h1>\n");
      return false;   
    }
    rb.debug.write("<h4>In 'makeLive' - B</h4>\n");
    // Load the draft version.
    var fileItDraft = rb.file.get({fileID:draftVersion.fileID, allData:0});
    if (!fileItDraft.getNextRow()) {
      // Wasn't really a draft = EXIT.
      rb.debug.error("Data inconsistency - there is no draft file object for web page");
      return false;
    }
    rb.debug.write("<h4>In 'makeLive' - C</h4>\n");
    var madeLive = false;

    // There is a draft version. Make new live file?
    if (liveVersion!=null || liveVersion.fileID>0) {
      // Already have a live version - replace the contents.
      rb.debug.write("<h4>In 'makeLive' - D</h4>\n");
      var fileIt = rb.file.get({fileID:liveVersion.fileID, allData:0});
      if (fileIt.getNextRow()) {
         // OK - file already exists. Change content and save.
         rb.debug.write("<h4>In 'makeLive' - E</h4>\n");
         fileIt.fileDef = fileItDraft.fileDef;

         if (rb.file.modify(fileIt)<0) {
            // rb.page.write("<h1>2 - failed to modify existing file content.</h1>\n");
            return false;
         }
         rb.debug.write("<h4>In 'makeLive' - F</h4>\n");

         madeLive = true;
      }
    }
    if (!madeLive) {
      // No live version. Create file and store.
      var fileID = rb.file.create({fileDef:fileItDraft.fileDef, name:fileItDraft.name, owner:fileItDraft.owner});
      if (fileID>=0) {
        this.fileRefs[0]={fileOrder:0, fileID:fileID};
        madeLive = true;
      }
    }
    if (madeLive) {
     // Update the 'madelive' time and increase the live count.
     // Need to save the document - but don't want to resave the content - I've just done that!
     this.lastLive = new Date;
     this.liveCount++;

     // Move over the 'draft' values to the 'live' values.
     this.livePageTitle  = this.pageTitle;
     this.livePageBanner = this.pageBanner;

     // The following requires the 'log' element to be installed. Only required for makeLive conditions.
     if (Log!=null)
       Log.newEntry('Web page "'+this.title+'" made live by "'+extauth.sessionUser+'", date = '+new Date());
     else
       rb.debug.write('Web page "'+this.title+'" made live by "'+extauth.sessionUser+'", date = '+new Date());

     item = this.makeItem();
     var res = (rb.catalogue.item.modify(item)>0);
     if (res && (this.state!='Draft'))
       this.publish();
    }
  }
  return res;
}

WebPage.prototype.getPageTitle = function(preview) {
  return preview ? this.pageTitle : this.livePageTitle;
}

WebPage.prototype.getPageBanner = function(preview) {
  return preview ? this.pageBanner : this.livePageBanner;
}
/********************************************************************************
WebPage.prototype.validateContent
Validate the contents of the webpage. Do the generic document checks then
do specific webpage checks. If docID is <0 then we're creating a new document
so make sure the path unique (not currently used).

Return null if the page contains valid data - return an error message otherwise.
********************************************************************************/
WebPage.prototype.validateContent = function(dataset) {

  if (dataset!=null)
    var msg = Doc.prototype.validateContent.call(this, dataset);

  if (msg==null) {
    // Generic tests are OK. Do we need to check the path for this object?
    if (this.ID==null || this.ID<0) {
      // Creating a new document - need to check that the path is unique.
      if (this.title==null || this.title=='')
        return 'Must specify path for this page.';

      // The title *must* start with a '/'
      if (this.title.indexOf('/')!=0)
        return 'The path must start with "/"';

      // Check for illegal characters in the pathname.
      if (this.title.search(/[^\w\/\.]/)>=0)
        return 'The path contains illeal characters. May only contain a-z, A-Z, 0-9 and underscore.';

      // *If* there is an existing page with this path then a new one *cannot* be created.
      var it   = rb.catalogue.item.get({idMkr:'N', ID:SysData.RootCats.WebPages, searchInDescription:true, keyword:[this.title], allLevels:false});
      if (it.getNextRow())
        return 'A page with that path already exists - choose a unique path for this document.';
    }                                       
  }
  return msg;
}
/********************************************************************************
WebPageIt - WebPage Iterator.
 ********************************************************************************/
/********************************************************************************
WebPageIt
Iterator for WebPages instance. This is the constructor. Doesn't do anything 
except at the moment.
********************************************************************************/
function WebPageIt(catIterator) {
    this.rb$Iterator = catIterator;
}

/********************************************************************************
* Force the iterator to be a subclass of the Event.
*********************************************************************************/
WebPageIt.prototype = new WebPage;

/********************************************************************************
WebPageIt.prototype.getNextRow
Overload for the iterator getNextRow method.
********************************************************************************/
WebPageIt.prototype.getNextRow = function (wantWrite) {
var res = false;
   
  do {
     do {
        res = this.rb$Iterator.getNextRow();
     } while (res && !Doc.isInstance(this.rb$Iterator));

     this.makeDoc(this.rb$Iterator);
  } while (res && (this.checkAccess()[test]==false));

  return res;
}
/********************************************************************************
WebPage.get
Create an iterator for WebPages.
********************************************************************************/
WebPage.get = function (data) {
var iterator;
var webPageIt;

   if (data==null)
      data = {ID:SysData.RootCats.WebPages, idMkr:'N', allLevels:1};

   iterator = rb.catalogue.item.get(data);
   webPageIt = new WebPageIt(iterator);
   return webPageIt;
}

/********************************************************************************
WebPage.getByPath
Search for a web-apge with the specified path. The path is stored in the
'description' field in the catalogue - and *all* pages ares stored directly under
the WebPages catalogue category.

The method returns *either* the page requested or 'null' if no such page exists.
********************************************************************************/
WebPage.getByPath = function (path, wantWrite, wantDraft, andContent, params, rtnAll) {
var page;

  if (SysData.RootCats!=null) {
     var it   = rb.catalogue.item.get({idMkr:'N', ID:SysData.RootCats.WebPages, searchInDescription:true, keyword:[path], allLevels:false});
     var test = (wantWrite ? 'write' : 'read');

     while (it.getNextRow()) {
       // It exists - is it visible to the current user?
       page = new WebPage(params,true,wantDraft);
       page.makeDoc(it);
    
       if (andContent==true)
         page.getContent();

       page.canRead = (page.checkAccess()[test]);
       if (page.canRead || rtnAll) {
         // Access allowed.
         return page;
       }
     }
  }
  return null;
}

