/*******************
WG
********************
Wrapper class for generic documents. Each news article is a catalogue item. If the
length of the article body is <Doc.maxItemLength characters it is stored in the stockData field - 
otherwise it is in a separate file.
********************/
function WG(input_data) {
var restored = false;
var it = null;

   this.name         = '';
   this.description  = '';
   this.customData   = {};
   this.privData     = {};
   this.chair        = '';         // Chair of the working group (uName).
   this.community    = -1;
   this.forum        = '';         // Name of the forum associated with this forum. All start with $$$WG$$$ in the title.
   this.ID           = -1;         // Of the catalogue node.
   this.created      = new Date();
   this.lastChange   = new Date();
   this.state        = 'proposed'; // Can be 'proposed', 'active', 'suspended'.
   this.membership   = '';         // 'open' anyone can join, 'admin' administratively , 'restrict' must belong to communtiy in this.communitySet.
   this.accessList = '';         // Listof communities/users used for membership verification if this.membership=='restrict'

   if (input_data == null)
      input_data = {};

   // dumpObj(input_data, "input_data");
   var id = (input_data.ID == null)?-1:input_data.ID;
   // rb.debug.write("<h1>id = "+id+"</h1>");
   // If there is an ID then try to read this catalogue item.
   var it;
       
   if (id>0) {
     it = rb.catalogue.category.get({ID:id, idMkr:'N', allLevels:false, nextLevel:false});
     if (it.getNextRow()) {
       // rb.debug.write("Found catalogue record<br>");
       this.makeWG(it);
     }
   }
   // Overlay input_data parts.
   for (var p in WG.props3) {
     if (input_data[p]) {
        this[p] = input_data[p];
     }
   }
}

WG.prototype.makeWG = function (item) {

   // Got the existing article. Load the contents.
   var dataSource;

   if (item.nodeData!=null) {
      dataSource = this.privData = item.nodeData.privData;
      this.customData = item.nodeData.customData;
   }

   for (var p=0;p<WG.props2.length;p++) {
      if (item[WG.props2[p].item]!=null) 
         this[WG.props2[p].wg] = item[WG.props2[p].item];
      else
         delete this[WG.props2[p].wg];
   }
   if (dataSource!=null) {
      for (var p in WG.props) {
         if (dataSource[p])
            this[p] = dataSource[p];
         else
            delete this[p];
      }
   }                       

   // Finally - if there is a state then save it so we can detect state changes on save.
   this.oldState = this.state;
} 

// Used between catalogue and WG
// a) Things from the nodeData record.
WG.props   = { chair:1, 
               forum:1,       // catalogue/member ID of author.
               created:1,     // Date item was created.
               lastChange:1,  // Date of last '.save()' operation
               state:1,
               community:1,
               membership:1,
               accessList:1,
               docNode:1,
               newsNode:1,
               eventNode:1
           };

// b) Things from the top level of the catalogue item.
WG.props2  = [    {wg:'name',       item:'name'},
                  {wg:'description',item:'description'},
                  {wg:'ID',         item:'nodeID'}];

// Used between rb$params and WG
WG.props3  = {name:1, description:1, lastChange:1, state:1, chair:1};

     this.docNode   = MakeCategory({name:'Documents', description:'WG documents'}, this.ID);
     this.newsNode  = MakeCategory({name:'Documents', description:'WG documents'}, this.ID);
     this.eventNode = MakeCategory({name:'Documents', description:'WG documents'}, this.ID);
/********************************************************************************
WG.prototype.makeItem()
Create data needed to be saved for this article. Returns an object with TWO
things in it.
   item:{}    - Catalogue item. Including ID if available.
   file:{}    - File object *if* article is too big for the catalogue.
********************************************************************************/
WG.prototype.makeItem = function () {
  var item = {nodeData:{}};

   if (this.privData==null)
      this.privData = {};

   // Copy top level properties.
   for (var p in WG.props) {
      if (this[p] != null) {
         this.privData[p] = this[p];
      }
   }

   // Copy top level properties.
   for (var p=0;p<WG.props2.length;p++) {
      if (this[WG.props2[p].wg]!=null)
         item[WG.props2[p].item] = this[WG.props2[p].wg];
   }
   item.nodeData.privData   = this.privData;
   item.nodeData.customData = this.customData;
   // dumpObj(this, "makeItem{wg}");
   return item;
}

/********************************************************************************
WG.prototype.create
Create the WG from the data provided. Have to do the following:
  a) Create a membership community.
  b) Create a discussion forum
  c) Create the WG record itself.
********************************************************************************/
WG.prototype.create = function() {

   // Set the modify time to 'now'.
   this.lastChange =
   this.created    = new Date();

   // Create the community.
   // dumpObj(this.makeItem(), "CatalogueNode");
   
   this.community = rb.contact.createCommunity({name:this.name, description:'WG:'+this.name});
   if (this.community<=0)
     // can't create the community.
     return -1;

   // Now create a new discussion forum.
   if (rb.page.env.ServerName().indexOf('home.net')<0) {
     ret = rb.message.group.createGroup('wg'+this.community+'.wg', this.name);
     if (ret==null) {
       // rb.debug.write("Couldn't create message group<br>\n");
       // couldn't create - delete the community and return.
       rb.contact.remove(this.community);
       return -1;
     }
   }
   this.forum = 'wg'+this.community+'.wg';

   // Now create the WG category node.
   var catNode = this.makeItem();
   this.ID = MakeCategory(catNode, SysData.RootCats.WG);

   if (this.ID>0) {
     // Create 'Documents', 'News' and 'Events' sub-nodes for this.
     this.docNode   = MakeCategory({name:'Documents', description:'WG documents'}, this.ID);
     this.newsNode  = MakeCategory({name:'News',      description:'WG documents'}, this.ID);
     this.eventNode = MakeCategory({name:'Events',    description:'WG documents'}, this.ID);
     this.save();
   }
   return this.ID;
}

/********************************************************************************
WG.prototype.save
Save the contents of the article after an edit. Reuse a file if one has already
been used - otherwise only float to a file if the content is now too big to
fit in the stockdata field of the catalogue item structure.
********************************************************************************/
WG.prototype.save = function () {
var item;

   if (this.ID==null || this.ID<0) {
      rb.debug.error("Attempt to save a WG with no ID attribute - probably not created!");
      return false;
   }

   this.lastChange = new Date();
   item = this.makeItem();

   // dumpObj(item, "catItem(save)");
   return (rb.catalogue.category.modify(item)>0);
}

/********************************************************************************
WgIt - Working Group Iterator.
 ********************************************************************************/
/********************************************************************************
WgIt
Iterator for WG information. This is the constructor. Doesn't do anything 
except at the moment.
********************************************************************************/
function WgIt(catIterator) {
    this.rb$Iterator = catIterator;
}

/********************************************************************************
* Force the iterator to be a subclass of the Event.
*********************************************************************************/
WgIt.prototype = new WG;

/********************************************************************************
WGIt.prototype.getNextRow
Overload for the iterator getNextRow method.
********************************************************************************/
WgIt.prototype.getNextRow = function () {
var res = false;
   
  res = this.rb$Iterator.getNextRow();
  if (res) {
    if (res.ID == SysData.RootCats.WG)
      res = this.rb$Iterator.getNextRow();
  }
  if (res) {
    // Load the member fields from the iterator.
    this.makeWG(this.rb$Iterator);
  }
  return res;
}
/********************************************************************************
WG.get
Create an iterator for working groups.
********************************************************************************/
WG.get = function (data) {
var iterator;
var memIt;

   if (data==null)
      data = {ID:SysData.RootCats.WG, idMkr:'N', nextLevel:1};

   iterator = rb.catalogue.category.get(data);
   it = new WgIt(iterator);

   return it;
}



/**************************
WG.getListToCommunity
Time saving device. When the user first clicks on the working group - cache the list 
in the session data. Return the session data list unless told to refresh.

Note that visibility rules apply - group only added if the user is allowed to see it
OR the 'allowAll' flag is set (eg for admin).
**************************/
WG.getListToCommunity = function(refresh) {
var sessData = rb.page.session.read();
var wgList;

  if (sessData.wgList!=null && !refresh && false) {
    // Make sure this is the same user for whom the list was built.
    if (sessData.wgList.user == extauh.sessionUser) {
      // rb.debug.write("WG.getListToCommunity - from <b>session data</b><br>");
      return sessData.wgList;
    }
  }
    
  var it = WG.get();
  var wg = {user:extauth.sessionUser, groups:[], target:'index.rhtm?rbcmd-view=yes'}

  var count = 0;
  while (count<20 && it.getNextRow()) {
    // Is this one visible?
    var visible=true;
    if (it.membership!='open') {
       // Check the communities.
      visible = _rb$extauth_isin(extauth.sessionUser, 'super,'+it.accessList)
    }

    if (visible) {
      count++;

      var name = it.name;

      // It it's too long - truncate it.
      var shortName;
      if (name.length>16)
        // Truncate it - used for memus.
         shortName = (name).substr(0,13)+'...';
      else
         shortName = name;

      wg.groups.push({name:it.name, sname:shortName, ID:it.ID});
    }
  }
  sessData.wgList = wg;
  // dumpObj(wg, "wg");
  rb.page.session.write(sessData);
  return wg;
}

WG.buildMenu = function(refresh) {
var wgList = WG.getListToCommunity(refresh);

  // Create menu items.
  var menuList = [];

  menuList.parent='wg';

  for (var i=0;i<wgList.groups.length;i++) {
    menuList.push({name:wgList.groups[i].sname, href:'/wg/'+wgList.target+'&ID='+wgList.groups[i].ID});
  }
  menuList.push({name:'<hr>', key:'dummy3xx'});

  // rbaddmenutree(menuList);
}
