// menu.js : Copyright 2002 YellowHawk Ltd (www.yellowhawk.co.uk)

// Script file that contains library routines for dynamic menu configuration. The system
// supports multiple independent, hierarchical menu structures - each held in a separate file
// object.
//
// This class requires a menu structure to be stored in a file object - the ID being passed
// in when the menu is created. It does not deal with how that file is referenced. In this
// way the system can deal with both system menus and customised personal menus.

// The 'Menu' class.
function DynMenu(id) {
  this.menuRoot = {};
  this.error    = false;
  this.changed  = false;
  this.id       = -1;

  // If there is an id then it is a reference to a file object containing the XML data.
  if (id!=null && id>0) {
    this.id = id;
    var it  = rb.file.get({fileID:id});
    if (it.getNextRow()) {
      this.buildMenu(it.fileDef.toString());
    }
  }
}

DynMenu.prototype.buildMenu = function(xmlSource) {
  // Want to build the XML representation of a menu tree from the source provided.
  this.parser = new XmlParser;

  if (!this.parser.build(xmlSource)) {
    // Error - shuldn't happen....
    for (var i=0;i<this.parser.errors.length;i++) {
      rb.debug.write(Util.doXmp(this.parser.errors)+'<br>');
    }
    rb.debug.error("Error in stored menu structure.");
  }
}

// Save the specified menu. There *must* be a ready existing file object.
DynMenu.prototype.save = function() {
  if (this.id>0) {
    rb.debug.write("<h1>SAVING modified menu structure content</h1>");
    var newContent = new Binary;
    newContent.set(this.parser.root.execute().toString());
    rb.file.modify({fileID:this.id, name:'Menu Structure', owner:'superuser', description:'Contents of the dynamic menu structure', fileDef:newContent});
  }
}

