// Email class. Maintains a 'work in progress' email in the session data area.
function Email(create) {
var sessData = rb.page.session.read();

  if (create || sessData.email==null) {
    // Initialise the session data area for this email.
    sessData.email = Email.NewEmail;
    rb.page.session.write(sessData);
  }
  this.subject = sessData.email.subject;
  this.body    = sessData.email.body;
  this.toList  = sessData.email.toList;
  this.ccList  = sessData.email.ccList;

  this.toStr = this.toListString();
  this.ccStr = this.ccListString();
}

Email.NewEmail = {toList:{}, ccList:{}, subject:'', body:''};

Email.prototype.clear = function () {
var sessData = rb.page.session.read();

  // Initialise the session data area for this email.
  sessData.email = Email.NewEmail;
  rb.page.session.write(sessData);

  this.subject = '';
  this.body    = '';
  this.toList  = {};
  this.ccList  = {};
}

Email.prototype.clearLists = function () {
  this.toStr = {};
  this.ccStr = {};
}


Email.prototype.save = function () {
  var sessData = rb.page.session.read();
  sessData.email = {subject:this.subject, body:this.body, toList:this.toList, ccList:this.ccList};
  rb.page.session.write(sessData);
}

Email.prototype.listToString = function (list) {
var str='', sep='';

  for (var i in list) {
    str += (sep+i);
    sep  = ',';
  }
  return str;
} 

Email.prototype.toListString = function() {
  return this.listToString(this.toList);
} 
Email.prototype.ccListString = function() {
  return this.listToString(this.ccList);
} 

// This method takes a string containing a list of recipients, breaks the list
// around commas. validates (simply) each entry and appends to the list specified.
Email.prototype.mergeListString = function(str,list) {
  var listArray = (str).split(/[,\s]+/);

  // rb.debug.write("<hr><h4>string is ["+str+"], length="+str.length+"</h4>\n");
  // dumpObj(listArray, "listArray");

  for (var i=0;i<listArray.length;i++) {
    if (listArray[i] && listArray[i]!='') {
      rb.debug.write("<h4>listArray["+i+"] is ["+listArray[i]+"], length="+listArray[i].length+"</h4>\n");
      list[listArray[i]] = true;
    }
  }
  rb.debug.write("<h4>string AT END is ["+str+"], length="+str.length+"</h4>\n");
  rb.debug.write("<hr>\n");
}
Email.prototype.addStringTO = function(str) {this.mergeListString(str, this.toList);}
Email.prototype.addStringCC = function(str) {this.mergeListString(str, this.ccList);}

// Process a 'to/cc' list. This method is used before sending a message to
// take each entry in the list and convert it to a valid email address. The
// implementation allows the list to include real email addresses (with '@' 
// in them) or uNames - in which case the membership component is used to
// translate the name into an address.
//
// The method returns an object containing two arrays - one of accepted 
// names, the other of uNames that could not be resolved (i.e. they are not 
// valid uNames.
Email.prototype.processList = function(addrList) {
  var res = {ok:[], err:[]};

  for (var addr in addrList) {
    // Is this an email address?
    if ((addr).indexOf('@')>=0) {
      // Assume it's an email address.
      rb.debug.write("'"+addr+"' is an email address<br>");
      res.ok.push(addr);
    }
    else {
      // Assume it's a uName and translate it.
      rb.debug.write("'"+addr+"' is a uName address<br>");
      var aMember = new Member({uName:addr});
      if (aMember.ID!=null && aMember.ID>0)
        res.ok.push(aMember.email);
      else
        res.err.push(addr);
    }
  }
  return res;
}

// Prepare the email to be sent. Return 'true' if the message is
// ready to be sent - return 'false' if there are errors.
Email.prototype.prepare = function() {
var res = false;

  this.toList_proc = this.processList(this.toList);
  this.ccList_proc = this.processList(this.ccList);

  res = (this.toList_proc.err.length==0 && this.ccList_proc.err.length==0);
  if (res) {
    this.to = this.toList_proc.ok;
    this.cc = this.ccList_proc.ok;
  }
  return res;
}
